/* global WE, top */

/**
 webEdition CMS
 *
 * $Rev: 7dc2e5379f69 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-10-16 13:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function Fileupload_sender_abstract(uploader) {
	this.uploader = uploader;

	this.doCommitFile = true;
	this.chunkSize = 256 * 1024;
	this.responseClass = 'we_fileupload_ui_base';
	this.typeCondition = [];
	this.maxUploadSize = 0;
	this.rpcUrl = '';
	this.callback = function () {
		this.uploader.doc.forms[0].submit();
	};
	this.cmdUploadFinished = '';
	this.replaceCmdUploadFinished = '';
	this.replaceNextCommand = '';
	this.dialogCallback = null;
	this.isCancelled = false;
	this.preparedFiles = [];
	this.uploadFiles = [];
	this.currentFile = -1;
	this.totalFiles = 0;
	this.totalWeight = 0;
	this.currentWeight = 0;
	this.currentWeightTag = 0;//FIXME: find better name
	this.isAutostartPermitted = false;
	this.partProperties = null;
	this.singleSuccessMessages = '';
	this.retry = 0;

	this.init = function (conf) {
		this.controller = this.uploader.controller; // on init all components are initialized
		this.view = this.uploader.view;
		this.imageEdit = this.uploader.imageEdit;
		this.utils = this.uploader.utils;
		this.typeCondition = conf.typeCondition ? conf.typeCondition : this.typeCondition;
		this.doCommitFile = conf.doCommitFile !== undefined ? conf.doCommitFile : this.doCommitFile;
		this.chunkSize = typeof conf.chunkSize !== 'undefined' ? (conf.chunkSize * 1024) : this.chunkSize;
		this.callback = conf.callback ? conf.callback : this.callback;
		this.cmdUploadFinished = conf.cmdUploadFinished ? conf.cmdUploadFinished : this.cmdUploadFinished;
		this.responseClass = conf.responseClass ? conf.responseClass : this.responseClass;
		this.dialogCallback = conf.dialogCallback ? conf.dialogCallback : this.dialogCallback;
		this.maxUploadSize = typeof conf.maxUploadSize !== 'undefined' ? conf.maxUploadSize : this.maxUploadSize;
		this.rpcUrl = conf.rpcUrl ? conf.rpcUrl : this.rpcUrl;
	};

	this.onload = function () {
		this.onload_sub();
	};

	this.onload_sub = function () {
		// to be overridden
	};

	this.resetSender = function () {
		this.preparedFiles = [];
		this.imageEdit.memorymanagerReregisterAll();
		this.uploadFiles = [];
		this.currentFile = -1;
		this.isCancelled = false;
		this.totalFiles = this.totalWeight = this.currentWeight = this.currentWeightTag = 0;
		this.view.setExternalProgress(0);
		this.singleSuccessMessages = '';
		// check for more props to reset!
	};

	this.prepareUpload = function () { // TODO: make this generic for use in importer too
		if (this.preparedFiles.length && typeof this.preparedFiles[0] === 'object' && this.preparedFiles[0] !== null && this.preparedFiles[0].isUploadable) {
			this.preparedFiles[0].fileNum = 0;
			this.preparedFiles[0].preparedFilesIndex = 0;
			this.uploadFiles = [this.preparedFiles[0]];
			this.totalWeight = this.preparedFiles[0].size;
			this.totalFiles = 1;
			this.currentWeight = 0;
			this.totalChunks = this.totalWeight / this.chunkSize;
			this.currentWeight = 0;
			this.currentWeightTag = 0;

			this.controller.processEvents(this.controller.ON.UPLOAD_START);//NEW

			return true;
		}

		return false;
	};

	this.getIsUploadableFiles = function () {
		for (var i = 0; i < this.preparedFiles.length; i++) {
			if (this.preparedFiles[i] && this.preparedFiles[i].isSizeOk && this.preparedFiles[i].isTypeOk) {
				return true;
			}
		}

		return false;
	};

	this.sendNextFile = function () {
		var cur, reader = null, cnt, editOptsLast, editOpts;

		if (this.uploadFiles.length > 0) {
			this.uploader.uploaderState = this.controller.STATE.UPLOADING;

			cur = this.uploadFiles[0];

			if (cur.isEditable) {
				editOptsLast = cur.img.processedOptions;//JSON.parse(JSON.stringify(cur.img.editOptions));
				this.imageEdit.setImageEditOptionsFile(cur);
				editOpts = cur.img.editOptions;

				if (editOpts.doEdit) {
					if (!cur.dataUrl || !cur.dataUrl.length ||
									editOpts.doEdit != editOptsLast.doEdit || // IMPORTANT: make config string: "1,w,1280,270,90" to compare!!
									editOpts.scaleWhat != editOptsLast.scaleWhat ||
									editOpts.scale != editOptsLast.scale ||
									editOpts.rotate != editOptsLast.rotate ||
									editOpts.quality != editOptsLast.quality) {

						// image is not yet edited or edited using options others than actually valid ones!
						cur.isEdited = false;
						this.preparedFiles[cur.preparedFilesIndex].img.callback = 'sendNextFile';
						this.preparedFiles[cur.preparedFilesIndex].tmpSize = this.preparedFiles[cur.preparedFilesIndex].size;

						// we process this image and call sendNextFile again!
						this.imageEdit.processSingleImage(this.preparedFiles[cur.preparedFilesIndex]);
						return;
					}

					// dataUrl of edited file exists: make dataArray, reinsert meta if any and go on:
					cur.dataArray = this.utils.dataURLToUInt8Array(cur.dataUrl);
					if (cur.type === 'image/jpeg' && cur.img.jpgCustomSegments) {
						cur.dataArray = this.imageEdit.jpgInsertSegment(cur.dataArray, cur.img.jpgCustomSegments);
					} else if (cur.type === 'image/png' && cur.img.pngTextChunks) {
						cur.dataArray = this.imageEdit.pngReinsertTextchunks(cur.dataArray, cur.img.pngTextChunks);
					}
				}
				cur.dataUrl = null;
			}

			this.currentFile = cur = this.uploadFiles.shift();
			cur.isUploadStarted = true;
			this.controller.processEvents(this.controller.ON.FILE_SEND, cur.index, cur);

			if (cur.size <= this.chunkSize && !cur.img.editOptions.doEdit) {
				this.sendNextChunk(false);
				return;
			}
			if (this.view.elems.fileSelect && this.view.elems.fileSelect.value) {
				this.view.elems.fileSelect.value = '';
			}

			if (cur.isEditable && cur.dataArray && cur.dataArray.length) {
				// we have an edited image width dataArray already prepared
				this.sendNextChunk(true);
				return;
			}
			// we have an image not to be edited, or other filetype
			reader = new FileReader();
			reader.onload = this.loadDataArrayAndSend.bind(this, cur);
			/*
			 reader.onload = function (e) {
			 cur.dataArray = new Uint8Array(e.target.result);
			 instance.sender.sendNextChunk(true);
			 };
			 */
			reader.readAsArrayBuffer(cur.file);

			return;
		}
		//all uploads done
		this.currentFile = null;
		if (this.uploader.getUploaderState === this.controller.STATE.UPLOADING) {
			this.uploader.uploaderState = this.controller.STATE.FINISHED;
		} else {
			this.uploader.uploaderState = this.controller.STATE.NOTREADY;
		}

		this.postProcess();

	};

	this.loadDataArrayAndSend = function (fileobj, e) {
		//fileobj = this.uploadFiles[0];
		fileobj.dataArray = new Uint8Array(e.target.result);

		// FIXME: fix longest side and recalculate total size for importer when file size is corrected!
		if (fileobj.size !== fileobj.dataArray.byteLength) {
			// FIXME: make function to recalculate all sizes => to be used in Fileupload_sender_import.prepareUpload() too
			var oldSize = fileobj.size;
			fileobj.size = fileobj.dataArray.byteLength;
			fileobj.totalParts = Math.ceil(fileobj.size / this.chunkSize);
			fileobj.lastChunkSize = fileobj.size % this.chunkSize;
			//in case the last segment is "0", the real size is chunkSize, since we don't have a segment containing the remaining contents
			fileobj.lastChunkSize = fileobj.lastChunkSize ? fileobj.lastChunkSize : this.chunkSize;

			this.totalWeight = this.totalWeight - oldSize + fileobj.size;
			this.totalChunks = this.totalWeight / this.chunkSize;
		}

		this.sendNextChunk(true);
	};

	this.sendNextChunk = function (split, resent) {//FIXME: why do we need this param???
		var oldPos, blob,
						cur = this.currentFile; // when using short syntax in line 1156 we must change some this to that = sender. FIXME: WHY?!

		if (this.isCancelled) {
			this.isCancelled = false;
			return;
		}
		if (resent) {
			this.retry++;
		} else {
			this.retry = 0;
		}
		if (!split) {
			this.sendChunk(cur.file, (cur.fileName ? cur.fileName : cur.file.name), cur.file.type, cur.size, 1, 1, '', cur.size);
			return;
		}
		if (cur.partNum < cur.totalParts || resent) {
			if (resent) {//this may fail on last chunk
				oldPos = (cur.partNum - 1) * this.chunkSize;
			} else {
				oldPos = cur.currentPos;
				cur.currentPos = oldPos + this.chunkSize;
				cur.partNum++;
			}
			blob = new Blob([cur.dataArray.subarray(oldPos, cur.currentPos)]);

			this.sendChunk(
							blob,
							(cur.fileName ? cur.fileName : cur.file.name),
							(cur.mimePHP !== 'none' ? cur.mimePHP : cur.file.type),
							(cur.partNum === cur.totalParts ? cur.lastChunkSize : this.chunkSize),
							cur.partNum,
							cur.totalParts,
							cur.fileNameTemp,
							cur.size
							);
		}
	};

	this.sendChunk = function (part, fileName, fileCt, partSize, partNum, totalParts, fileNameTemp, fileSize) {
		var
						xhr = new XMLHttpRequest(),
						fd = new FormData(),
						fsize = fileSize || 1,
						form = this.uploader.doc.we_form,
						cur = this.currentFile;

		this.partProperties = {
			partSize: partSize,
			partNum: partNum,
			totalParts: totalParts
		};

		fileCt = fileCt ? fileCt : 'text/plain';
		fd.append('fileinputName', this.uploader.fieldName);
		fd.append('doCommitFile', this.doCommitFile);
		fd.append('genericFilename', this.uploader.genericFilename);
		fd.append('weResponseClass', this.responseClass);
		fd.append('wePartNum', partNum);
		fd.append('wePartCount', totalParts);
		fd.append('wePartSize', partSize);
		fd.append('weRetry', this.retry);
		fd.append('weFileNameTemp', fileNameTemp);
		fd.append('weFileSize', fsize);
		fd.append('weFileName', fileName);
		fd.append('weFileCt', fileCt);
		fd.append(this.currentFile.field !== undefined ? this.currentFile.field : this.uploader.fieldName, part, fileName);//FIXME: take fieldname allways from cur!

		fd.append('fu_doc_focusX', cur.img.focusX);
		fd.append('fu_doc_focusY', cur.img.focusY);
		fd.append('fu_doc_importMetadata', (form.fu_doc_importMetadata ? form.fu_doc_importMetadata.value : form.fu_doc_importMetadata_ctrl.value));

		fd = this.appendMoreData(fd);
		xhr.onreadystatechange = this.sendChunkAjaxCallback.bind(this);
		//FIXME: use XMLHttpRequestUpload?!
		xhr.timeout = 12000;
		xhr.open('POST', this.rpcUrl, true);
		xhr.send(fd);
	};

	this.sendChunkAjaxCallback = function (e) {
		var xhr = e.target;

		switch (xhr.readyState) {
			case 0://unsent
			case 1://opened
			case 2://headers received
			case 3://loading
				break;
			case 4:
				switch (xhr.status) {
					case 0://e.g. network error
						if (this.retry > 10) {
							WE().t_e('Uploader failed for more than 10 times');
							//do cancel
							return;
						}
						//extend retry interval
						window.setTimeout(this.sendNextChunk.bind(this, true, true), (this.retry * 3000));
						break;
					case 200:
						var resp = '';

						try {
							resp = JSON.parse(xhr.responseText);
						} catch (exception) {
							WE().t_e('Response unable to parse: ' + xhr.responseText);
							throw exception;
						}

						resp = resp.DataArray && resp.DataArray.data ? resp.DataArray.data : resp;
						this.processResponse(resp, this.partProperties);
						break;
					default:
						WE().t_e("Uploader received bad status from server: " + xhr.status);
				}
				break;
		}
	};

	this.appendMoreData = function (fd) {
		return fd;
		/*
		 parentID
		 sameName
		 */
	};

	this.processResponse = function (resp, args) {
		if (!this.isCancelled) {
			var cur = this.currentFile;

			cur.fileNameTemp = resp.fileNameTemp;
			cur.mimePHP = resp.mimePhp;
			cur.currentWeightFile += args.partSize;
			this.currentWeight += args.partSize;

			switch (resp.status) {
				case 'continue':
					this.controller.processEvents(this.controller.ON.CHUNK_RESPONSE_OK, cur.index, cur);
					this.sendNextChunk(true);
					return;
				case 'success':
					this.currentWeightTag = this.currentWeight;
					cur.dataArray = null;
					cur.dataUrl = null;
					if (this.imageEdit.IS_MEMORY_MANAGMENT) {
						this.imageEdit.memorymanagerUnregister(cur);
					}
					this.controller.processEvents(this.controller.ON.CHUNK_RESPONSE_OK, cur.index, cur);
					this.controller.processEvents(this.controller.ON.FILE_RESPONSE_OK, cur.index, cur);
					this.doOnFileFinished(resp);
					this.singleSuccessMessages += resp.messageSingleFile ? resp.messageSingleFile : '';
					if (this.uploadFiles.length !== 0) {
						this.sendNextFile();
					} else {
						this.postProcess(resp);
					}
					return;
				case 'retransmit'://chunksize did not match at server
					if (this.retry < 10) {
						this.sendNextChunk(true, true);
						return;
					}
					//falls through
				case 'failure':
					this.currentWeight = this.currentWeightTag + cur.size;
					this.currentWeightTag = this.currentWeight;
					cur.dataArray = null;
					cur.dataUrl = null;
					if (this.imageEdit.IS_MEMORY_MANAGMENT) {
						this.imageEdit.memorymanagerUnregister(cur);
					}

					cur.failedRequest = resp.message;
					this.controller.processEvents(this.controller.ON.CHUNK_RESPONSE_NOK, cur.index, cur);

					if (this.uploadFiles.length !== 0) { // must be importer
						this.sendNextFile();
					} else {
						this.isCancelled = true;
						this.postProcess(resp);
					}
					return;
				default :
					return;
			}
		}
	};

	this.doOnFileFinished = function () {
		//to be overridden
	};

	this.postProcess = function (resp) {
		//to be overriden
	};

	this.cancel__ = function () {
		this.isCancelled = true;
		if (this.view.reloadOpener) {
			this.view.reloadOpener();
		}
		this.view.showMessageCancel(this.singleSuccessMessages);

		var cur = this.currentFile;
		this.controller.processEvents(this.controller.ON.UPLOAD_CANCEL, cur.index, cur);
	};
}

function Fileupload_sender_base(uploader) {
	Fileupload_sender_abstract.call(this, uploader);

	this.uploader = uploader;
	this.totalWeight = 0;

	this.postProcess = function (resp) {
		var cur = this.currentFile;
		var form = this.uploader.doc.forms.we_form;
		form.elements.weFileNameTemp.value = cur.fileNameTemp;
		form.elements.weFileCt.value = cur.mimePHP;
		form.elements.weFileName.value = cur.file.name;

		this.view.setStateItem(this.controller.STATE.ITEM_UPLOADED, 0);
		this.view.setStateGUI(this.controller.STATE.FINISHED);
		this.uploader.uploaderState = this.controller.STATE.FINISHED;

		if (this.cmdUploadFinished || this.replaceCmdUploadFinished) {
			var cmd = this.replaceCmdUploadFinished ? this.replaceCmdUploadFinished : this.cmdUploadFinished;
			var tmp = cmd.split(',');
			tmp.splice(1, 0, resp);
			window.setTimeout(this.controller.fireCmd, 100, [this.uploader.win, tmp]);
			//window.setTimeout(this.controller.fireCmd, 500, [this.uploader.win, ['close']]);
		}
	};
}
Fileupload_sender_base.prototype = Object.create(Fileupload_sender_abstract.prototype);
Fileupload_sender_base.prototype.constructor = Fileupload_sender_base;

function Fileupload_sender_bindoc(uploader) {
	Fileupload_sender_abstract.call(this, uploader);

	this.uploader = uploader;
	this.totalWeight = 0;

	this.appendMoreData = function (fd) {
		var form = this.uploader.doc.we_form;

		if (uploader.uiType === 'wedoc') {
			fd.append('we_transaction', form.we_transaction.value);

			return fd;
		}

		fd.append('fu_file_sameName', form.fu_file_sameName.value);
		fd.append('fu_file_parentID', form.fu_file_parentID.value);
		fd.append('fu_file_parentDir', form.fu_file_parentDir.value);
		fd.append('fu_doc_categories', (form.fu_doc_categories ? form.fu_doc_categories.value : ''));
		fd.append('fu_doc_isSearchable', form.fu_doc_isSearchable.value);
		fd.append('fu_doc_thumbs', form.fu_doc_thumbs.value);
		fd.append('fu_doc_alt', form.fu_doc_alt.value);
		fd.append('fu_doc_title', form.fu_doc_title.value);

		return fd;
	};

	this.doOnFileFinished = function (resp) {};

	this.postProcess = function (resp) {
		var win = this.uploader.win;

		this.preparedFiles = [];
		this.uploader.uploaderState = this.controller.STATE.FINISHED;

		if (uploader.uiType !== 'wedoc') {
			if (resp.status === 'failure') {
				this.resetSender();
			} else {
				this.currentFile = null;

				for (var k in resp.weDoc) {
					resp[k] = resp.weDoc[k];
				}

				var tmp = this.cmdUploadFinished.split(',');
				tmp.splice(1, 0, resp);
				window.setTimeout(this.controller.fireCmd, 100, [win.opener, tmp]);
				window.setTimeout(this.controller.fireCmd, 500, [this.uploader.win, ['close']]);

				//reload main tree and close!
				//or weFileUpload_instance.reset(); and let cmdUploadFinished close uploader!
			}
		} else if (resp.status === 'success') {
			this.currentFile = null;
			if (WE(true)) {
				win.we_cmd('update_file');
				WE().layout.we_setPath(null, resp.weDoc.path, resp.weDoc.text, 0, "published");
			}
		}
	};
}
Fileupload_sender_bindoc.prototype = Object.create(Fileupload_sender_abstract.prototype);
Fileupload_sender_bindoc.prototype.constructor = Fileupload_sender_bindoc;

function Fileupload_sender_import(uploader) {
	Fileupload_sender_abstract.call(this, uploader);

	this.isCancelled = false;
	this.totalChunks = 0; //FIXME: apply consistent terminology to differ between currentfile and all files: and make it part of abstract

	this.prepareUpload = function (rePrepare) {
		// when currently first file (we use shift!) has been reedited just before sending first chunk,
		// we must recalculat totalWeight and set external progress
		// FIXME: make separate fn recalculateTotalWeight()
		if (rePrepare) {
			var cur = this.currentFile;
			this.totalWeight = this.totalWeight - this.uploadFiles[0].tmpSize + this.uploadFiles[0].size;
			this.totalChunks = this.totalWeight / this.chunkSize;
			this.controller.processEvents(this.controller.ON.CHUNK_RESPONSE_OK, cur.index, cur);

			return true;
		}

		if (this.currentFile === -1) {
			this.uploadFiles = [];
			for (var i = 0, c = 0; i < this.preparedFiles.length; i++) {
				if (typeof this.preparedFiles[i] === 'object' && this.preparedFiles[i] !== null && this.preparedFiles[i].isUploadable) {
					this.preparedFiles[i].fileNum = c++;
					this.preparedFiles[i].preparedFilesIndex = i;
					this.uploadFiles.push(this.preparedFiles[i]);
					this.totalWeight += this.preparedFiles[i].size;
				}
			}
			this.totalFiles = this.uploadFiles.length;

			if (this.totalFiles > 0) {
				this.currentWeight = 0;
				this.totalChunks = this.totalWeight / this.chunkSize;
				this.currentWeight = 0;
				this.currentWeightTag = 0;
				this.controller.processEvents(this.controller.ON.UPLOAD_START);//NEW

				return true;
			}
		}
		return false;
	};

	this.appendMoreData = function (fd) {
		var form = this.uploader.doc.we_form;
		var cur = this.currentFile;

		fd.append('weFormNum', cur.fileNum + 1);
		fd.append('weFormCount', this.totalFiles);
		fd.append('we_cmd[0]', 'import_files');
		fd.append('step', 1);

		fd.append('fu_file_sameName', form.fu_file_sameName.value);
		fd.append('fu_file_parentID', form.fu_file_parentID.value);
		fd.append('fu_doc_categories', (form.fu_doc_categories ? form.fu_doc_categories.value : ''));
		fd.append('fu_doc_isSearchable', form.fu_doc_isSearchable.value);
		fd.append('fu_doc_thumbs', form.fu_doc_thumbs.value);

		return fd;
	};

	this.postProcess = function (resp) {
		this.view.reloadOpener();
		this.uploader.uploaderState = this.controller.STATE.FINISHED;

		this.view.setExternalProgress(100);
		this.view.setExternalProgressText('progress_title', '');
		WE().util.showMessage(resp.completed.message, WE().consts.message.WE_MESSAGE_INFO, this.uploader.win);

		if (this.cmdUploadFinished || this.replaceCmdUploadFinished) {
			var cmd = this.replaceCmdUploadFinished ? this.replaceCmdUploadFinished : this.cmdUploadFinished;
			if (cmd) {
				var tmp = cmd.split(',');
				tmp.splice(1, 0, resp);
				window.setTimeout(this.controller.fireCmd, 100, [top, tmp]);
			}
			this.uploader.reset();
			window.setTimeout(this.controller.fireCmd, 1000, [this.uploader.win, ['close']]);
		}
		window.setTimeout(this.uploader.reset.bind(this.uploader), 1000);


	};
}
Fileupload_sender_import.prototype = Object.create(Fileupload_sender_abstract.prototype);
Fileupload_sender_import.prototype.constructor = Fileupload_sender_import;
