<?php

/**
 * webEdition CMS
 *
 * $Rev: fcddb3030f18 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2019-11-20 20:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_base_country{

	const TERRITORY = 'territory';
	const REGION = 'region';
	const LANGUAGE = 'language';
	const SCRIPT = 'script';
	const MONTH = 'months';
	const DAY = 'days';

	static $last = [];

	private static function loadLang($langcode){
		$file = WE_INCLUDES_PATH . 'country/we_country_' . $langcode . '.class.php';
		if(!file_exists($file)){
			echo 'no file' . $file;
			return false;
		}

		include_once($file);
		return true;
	}

	public static function getTranslation($countrykey, $type, $langcode){
		list($langcode) = explode('_', strtolower($langcode), 2);
		self::loadLang($langcode);
		$class = 'we_country_' . $langcode;
		return empty($class::DATA[$type][$countrykey]) ? '' : $class::DATA[$type][$countrykey];
	}

	public static function getTranslationList($type, $langcode){
		list($langcode) = explode('_', strtolower($langcode), 2);
		self::loadLang($langcode);
		$class = 'we_country_' . $langcode;
		return empty($class::DATA[$type]) ? '' : $class::DATA[$type];
	}

	public static function dateformat($langcode, DateTime $date, $format){
		list($langcode) = explode('_', strtolower($langcode), 2);

		$months = self::getTranslationList(self::MONTH, $langcode);
		$days = self::getTranslationList(self::DAY, $langcode);

		$dat = $date->format(strtr($format, [
			'D' => "\dD\d", //Mon bis Sun
			'l' => "\lD\l", //Sunday bis Saturday
			'F' => "\\fn\\f", //January bis December
			'M' => '\mn\m', //Jan bis Dec
		]));

		$wd = $date->format('D');
		$mon = $date->format('n');

		return strtr($dat, [
			'd' . $wd . 'd' => $days['abbreviated'][strtolower($wd)], //Mon bis Sun
			'l' . $wd . 'l' => $days['wide'][strtolower($wd)], //Sunday bis Saturday
			'f' . $mon . 'f' => $months['wide'][$mon], //January bis December
			'm' . $mon . 'm' => $months['abbreviated'][$mon], //Jan bis Dec
				]
		);
	}

	public static function getWELangs(){
		return [
			'de' => 'Deutsch',
			'en' => 'English',
			'nl' => 'Dutch',
			'fi' => 'Finnish',
			'ru' => 'Russian',
			'es' => 'Spanish',
			'pl' => 'Polish',
			'fr' => 'French'
		];
	}

}
