<?php
/**
 * webEdition CMS
 *
 * $Rev: 79d4fb51d5b4 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-11-24 14:12 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/** the parent class of storagable webEdition classes */
abstract class we_contents_base{
	//constants for retrieving data from DB

	const LOAD_MAIN_DB = 0; //default DB
	const LOAD_TEMP_DB = 1; //temporary saved files
	const LOAD_SCHEDULE_DB = 3; //FIXME: remove this. We should load all data from temp table
	//constants define where to write document (guess)
	const SUB_DIR_NO = '-';
	const SUB_DIR_YEAR = 'y';
	const SUB_DIR_YEAR_MONTH = 'ym';
	const SUB_DIR_YEAR_MONTH_DAY = 'ymd';
	const ETYPE_UNKNOWN = 'ALLKEYS';
	const ETYPE_NUMBER = 'bdid';
	const ETYPE_TEXT = 'dat';
	const INTERNAL_VAR = 'vars';
	const MAX_INDEX_SIZE = 65535;

	/* Name of the class => important for reconstructing the class from outside the class */
	var $ClassName = __CLASS__;
	/* In this array are all storagable class variables */
	var $persistent_slots = ['ClassName', 'Name', 'ID', 'Table', 'wasUpdate', 'InWebEdition'];
	/* Name of the Object that was createt from this class */
	var $Name = '';

	/* ID from the database record */
	var $ID = 0;

	/* database table in which the object is stored */
	var $Table = '';
	public $LangLinks = [];
	public $Language = '';

	/* Database Object */
	protected $DB_WE;

	/* optional ID for being able to manipulate the ID for DB inserts, requires a getter and setter */
	protected $insertID = 0;

	/* Flag which is set when the file is not new */
	var $wasUpdate = false;
	public $InWebEdition = false;
	var $PublWhenSave = 1;
	var $IsTextContentDoc = false;
	var $fileExists = 1;
	protected $errMsg = '';
	protected $warningMsg = [];
	protected $noticeMsg = [];
	public $View = null;
	protected $ignoreLangLinkErr = false;

	/* Constructor */

	public function __construct(){
		$this->Name = md5(uniqid(__FILE__, true));
		$this->ClassName = get_class($this); //$this is different from self!
		$this->DB_WE = new DB_WE();
		if((isWE() || defined('WE_SIDEBAR')) && !empty($GLOBALS['we']['WE_SAVE_STATIC'])){
			$this->InWebEdition = true;
			//FIXME: add in 8.1
			//$this->View = we_view_base::getView($this);
		}
	}

	//Overwrite
	public function we_new(){

	}

	//Overwrite
	abstract public function we_initSessDat(array $sessDat);

	/* Intialize the class. If $sessDat (array) is set, the class will be initialized from this array */

	function init($sessDat = ''){
		$this->we_new();
	}

	/* set the protected variable insertID */

	function setInsertID($insertID){
		$this->insertID = $insertID;
	}

	/* get the protected variable insertID */

	function getInsertID(){
		return $this->insertID;
	}

	/* returns the url $in with $we_transaction appended */

	public static function url($in){
		return $in . (strpos($in, '?') !== false ? '&' : '?') . 'we_transaction=' . $GLOBALS['we_transaction'];
	}

	/* returns the code for a hidden " we_transaction-field" */

	public static function hiddenTrans(){
		return we_html_element::hidden('we_transaction', $GLOBALS['we_transaction']);
	}

	/* must be overwritten by child */

	abstract function saveInSession(&$save, $toFile = false);

	/* creates a color field. when user clicks, a colorchooser opens. Data that will be stored at the $elements Array */

	function formColor($width, $name, $type = 'txt', $height = 18, $isTag = false){
		$value = $this->getElement($name);
		if(!$isTag){
			$width -= 4;
		}
		$formname = 'we_' . $this->Name . '_' . $type . '[' . $name . ']';
		$out = we_html_element::hidden($formname, $this->getElement($name)) .
			we_html_element::span([
				'style' => 'display:block;width:' . $width . 'px;height:' . $height . 'px;background-color: ' . ($value ?: 'inherit') . ';border:1px solid black;',
				'onclick' => "setScrollTo();we_cmd('openColorChooser','" . $formname . "',document.we_form.elements['" . $formname . "'].value);"
				], '&nbsp;');
		return g_l('weClass', '[' . $name . ']', true) !== false ? we_html_tools::formElementTable(g_l('weClass', '[' . $name . ']'), $out) : $out;
	}

	function formTextInput($elementtype, $name, $text, $maxlength = '', array $attribs = []){
		if(!$elementtype){
			$ps = $this->$name;
		}
		return we_html_tools::formElementTable($text, we_html_tools::textInput(($elementtype ? ('we_' . $this->Name . '_' . $elementtype . '[' . $name . ']') : ('we_' . $this->Name . '_' . $name)), ($elementtype ? $this->getElement($name) : $ps), $maxlength, $attribs));
	}

	function formInputField($elementtype, $name, $text, $width, $maxlength = '', array $attribs = []){
		list($realName) = explode('#', $name);
		if(!$elementtype){
			$ps = $this->$realName;
		}
		return we_html_element::div(['style' => "position:relative"], $text .
				we_html_element::br() .
				we_html_tools::textInput('we_' . $this->Name . '_' . ($elementtype ? $elementtype . '[' . $name . ']' : $name), ($elementtype && ($elVal = $this->getElement($realName)) ? $elVal : (isset($GLOBALS['meta'][$realName]) ? $GLOBALS['meta'][$realName]['default'] : (isset($ps) ? $ps : '') )), $maxlength, $attribs, isset($attribs['type']) ? $attribs['type'] : 'txt', $width));
	}

	protected function formTextArea($elementtype, $name, $text, $rows = 0, $cols = 0, array $attribs = [], $optional = ''){
		return we_html_element::div(['style' => "position:relative"], $text .
				we_html_element::br() .
				self::htmlTextArea(($elementtype ? ('we_' . $this->Name . '_' . $elementtype . '[' . $name . ']') : ('we_' . $this->Name . '_' . $name)), $rows, $cols, $this->getElement($name), $attribs) .
				$optional
		);
	}

	protected function formSelectFromArray($elementtype, $name, array $vals, $text, $size = 1, $multiple = false, array $attribs = []){
		return we_html_tools::formElementTable($text, we_html_tools::select('we_' . $this->Name . '_' . ($elementtype ? $elementtype . '[' . $name . ']' : $name), $vals, $size, ($elementtype ? [
					$this->getElement($name)] : $this->$name), $multiple, $attribs));
	}

	static function htmlTextArea($name, $rows = 10, $cols = 30, $value = '', array $attribs = []){
		return we_html_element::textArea(array_merge(['name' => trim($name),
				'class' => 'defaultfont wetextarea',
				($rows ? 'rows' : '') => abs($rows),
				($cols ? 'cols' : '') => abs($cols),
					], $attribs
				), ($value ? (oldHtmlspecialchars($value)) : ''));
	}

	/* creates a select field for entering Data that will be stored at the $elements Array */

	protected function formSelectElement($width, $name, $values, $type = 'txt', $size = 1, array $attribs = []){
		return we_html_tools::formElementTable(g_l('weClass', '[' . $name . ']'), we_html_tools::select('we_' . $this->Name . '_' . $type . '[' . $name . ']', $values, $size, $this->getElement($name), false, array_merge([
					'class' => 'defaultfont',
					'width' => $width,
						], $attribs))
		);
	}

	/* creates a text-input field for entering Data that will be stored at the $elements Array and shows information from another Element */

	protected function formInput2($width, $name, $size, $type = 'txt', array $attribs = [], $infoname = ''){
		$infotext = $infoname ? ' (' . (g_l('weClass', '[' . $infoname . ']', true) ?: $infoname) . ': ' . $this->getElement($infoname) . ')' : '';
		list($origname) = explode('#', $name);
		return $this->formInputField($type, $name, (g_l('weClass', '[' . $origname . ']', true) ?: $origname) . $infotext, $width, '', $attribs);
	}

	protected function formSelect2($width, $name, $table, $sqlFrom, $text, $sqlTail = '', $size = 1, $selectedIndex = '', $multiple = false, $onChange = '', array $attribs = [
		], $precode = '', $postcode = '', $firstEntry = '', $gap = 20){
		$vals = [];
		if($firstEntry){
			$vals[$firstEntry[0]] = $firstEntry[1];
		}
		//FIXME: remove $sqlFrom or val/txt
		$vals = $vals + $this->DB_WE->getAllFirstq('SELECT ' . $sqlFrom . ' FROM ' . $table . ' WHERE ' . $sqlTail, false);

		$vals = we_html_tools::groupArray($vals, false, 1);
		$myname = 'we_' . $this->Name . '_' . $name;

		$ps = $this->$name;

		$pop = we_html_tools::select($myname . ($multiple ? 'Tmp' : ''), $vals, $size, $ps, $multiple, array_merge(['onchange' => $onChange . ($multiple ? ";var we_sel='';for(i=0;i<this.options.length;i++){if(this.options[i].selected){we_sel += (this.options[i].value + ',');};};if(we_sel){we_sel=we_sel.substring(0,we_sel.length-1)};this.form.elements['" . $myname . "'].value=we_sel;" : '')
					], $attribs), true, $width);

		if($precode || $postcode){
			$pop = '<table class="default"><tr>' . ($precode ? ('<td style="padding-right:' . $gap . 'px;">' . $precode . '</td>') : '') . '<td>' . $pop . '</td>' . ($postcode ? ('<td>' . $postcode . '</td>') : '') . '</tr></table>';
		}
		return ($multiple ? we_html_element::hidden($myname, $selectedIndex) : '') . we_html_tools::formElementTable($text, $pop);
	}

	protected function formSelect4($width, $name, $table, $sqlFrom, $text, $sqlTail = '', $size = 1, $selectedIndex = '', $multiple = false, $onChange = '', array $attribs = [
		], $precode = '', $postcode = '', array $firstEntry = [], $gap = 20){
		$vals = [];
		if($firstEntry){
			$vals[$firstEntry[0]] = $firstEntry[1];
		}
		//array_merge would change the numeric order of keys!
		$vals = $vals + $this->DB_WE->getAllFirstq('SELECT ' . $sqlFrom . ' FROM ' . $this->DB_WE->escape($table) . ' WHERE ' . $sqlTail, false);

		$myname = 'we_' . $this->Name . '_' . $name;

		$pop = we_html_tools::select($myname, $vals, $size, $selectedIndex, $multiple, array_merge(['onchange' => $onChange], $attribs), true, $width);
		if($precode || $postcode){
			$pop = '<table class="default"><tr>' . ($precode ? ('<td style="padding-right:' . $gap . 'px;">' . $precode . '</td>') : '') . '<td>' . $pop . '</td>' . ($postcode ? ('<td>' . $postcode . '</td>') : '') . '</tr></table>';
		}
		return we_html_tools::formElementTable($text, $pop);
	}

	public function initByID($ID, $Table = FILE_TABLE, $from = self::LOAD_MAIN_DB){
		$this->ID = intval($ID);
		$this->Table = ($Table ?: FILE_TABLE);
		$this->we_load($from);
		//$GLOBALS['we_ID'] = intval($ID); //FIXME: check if we need this !
		//$GLOBALS['we_Table'] = $this->Table;
		// init Customer Filter !
		$this->initWeDocumentCustomerFilterFromDB();
	}

	/**
	 * inits weDocumentCustomerFilter from db regarding the modelId
	 * is called from "we_textContentDocument::we_load"
	 * @see we_contents_textContentDocument::we_load
	 */
	protected function initWeDocumentCustomerFilterFromDB(){
		if(isset($this->documentCustomerFilter) && we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->documentCustomerFilter = we_customer_documentFilter::getFilterByIdAndTable($this->ID, $this->Table);
		}
	}

	public function we_load($from = self::LOAD_MAIN_DB){
		$this->getPersistentSlotsFromDB();
	}

	public function we_save($resave = 0){
		$this->wasUpdate = $this->ID > 0;
		return $this->savePersistentSlotsToDB();
	}

	public function we_delete(){
		we_base_delete::deleteEntry($this->ID, $this->Table, true, false, $this->DB_WE);
	}

	public function we_publish(/* $DoNotMark = false, $saveinMainDB = true */){
		return true; // overwrite
	}

	public function we_unpublish(/* $DoNotMark = false */){
		return true; // overwrite
	}

	public function we_republish(){
		return true;
	}

	protected function setElementsFromHTTP(){
		if($_REQUEST){
			// do not set REQUEST VARS into the document
			switch(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0)){
				case 'switch_edit_page':
					if(we_base_request::_(we_base_request::STRING, 'we_cmd', false, 3)){
						return true;
					}
					break;
				case 'save_document':
					if(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 7) === 'save_document'){
						return true;
					}
					break;
			}

			$regs = [];
			foreach($_REQUEST as $n => $v){
				if(preg_match('#^we_' . preg_quote($this->Name, '#') . '_([^\[]+)$#', $n, $regs)){
					$this->setPersistentSlot($regs[1], $v);
				}
			}
		}
	}

	protected function setPersistentSlot($name, $value){
		if(in_array($name, $this->persistent_slots)){
			$this->$name = $value;
		}
	}

	protected function getPersistentSlotsFromDB($felder = '*'){
		$fields = getHash('SELECT ' . $felder . ' FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID), $this->DB_WE);
		if($fields){
			foreach($fields as $k => $v){
				if($k && in_array($k, $this->persistent_slots)){
					$this->{$k} = $v;
				}
			}
		} else {
			$this->fileExists = 0;
		}
	}

	protected function savePersistentSlotsToDB(array $felder = []){
		$tableInfo = $this->DB_WE->metadata($this->Table, we_database_base::META_NAME);
		$feldArr = $felder ? $felder : $this->persistent_slots;
		$fields = [];
		if(!$this->wasUpdate && $this->insertID && f('SELECT 1 FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->insertID) . ' LIMIT 1', '', $this->DB_WE)){
			return false;
		}
		foreach($tableInfo as $fieldName){
			if(in_array($fieldName, $feldArr)){
				$val = isset($this->$fieldName) ? $this->$fieldName : '';

				switch($fieldName){
					case 'ID':
						if(!$this->wasUpdate && $this->insertID){//for Apps to be able to manipulate Insert-ID
							$fields['ID'] = $this->insertID;
							$this->insertID = 0;
						}
						break;
					case 'Filehash':
						$fields['Filehash'] = $val !== null ? sql_function('x\'' . $val . '\'') : null;
						break;
					default:
						$fields[$fieldName] = $val;
				}
			}
		}
		if($fields){
			$where = ($this->wasUpdate || $this->ID ? ' WHERE ID=' . intval($this->ID) : '');
			$ret = (bool) ($this->DB_WE->query(($where ? 'UPDATE ' : 'INSERT INTO ') . $this->DB_WE->escape($this->Table) . ' SET ' . we_database_base::arraySetter($fields) . $where));
			$this->ID = ($this->ID ?: $this->DB_WE->getInsertId());
			return $ret;
		}
		return false;
	}

	function setDocumentControlElements(){
		//	function is overwritten in we_webEditionDocument
	}

	function executeDocumentControlElements(){
		//	function is overwritten in we_webEditionDocument
	}

	function isValidEditPage($editPageNr){
		return (is_array($this->EditPageNrs) ?
			in_array($editPageNr, $this->EditPageNrs) :
			false);
	}

	/** this function returns true, if anything is to be done for langlinks,
	 * e.g. a changed link or the document language has changed */
	protected function checkLangLinksHaveChanged(){
		$ret = 0;
		$oldLang = f('SELECT Language FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID));
		if($oldLang !== $this->Language){
			$ret = 1;
		}
		ksort($this->LangLinks);
		$langlinks = array_filter($this->LangLinks);

		$langArray = $this->DB_WE->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' ll WHERE ll.DocumentTable="' . stripTblPrefix($this->Table) . '" AND ll.DID=' . intval($this->ID) . ' ORDER BY Locale', false);
		$ret |= ((count($langlinks) > count($langArray) ? empty(array_diff_assoc($langlinks, $langArray)) : empty(array_diff_assoc($langArray, $langlinks))) ? 0 : 2);
		return $ret;
	}

	/**
	 * update lang link table with the changed document lang, if possible
	 */
	protected function saveLangLinkDocLang(){
		//update locale, if possible for remote langs
		$this->DB_WE->query('UPDATE IGNORE ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
				'Locale' => $this->Language
			]) . ' WHERE
DocumentTable="' . stripTblPrefix($this->Table) . '" AND LDID=' . intval($this->ID));
	}

	/**
	 * Save all Langlink fields set in the gui to the table, and make backlinks if possible
	 */
	protected function saveLangLinkDocuments($withBackLink = true){
		$langlinks = array_filter($this->LangLinks);
		//delete unused
		$langs = array_keys($langlinks);
		$docTable = stripTblPrefix($this->Table);
		//get links to delete
		$allDels = $this->DB_WE->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND DID=' . intval($this->ID) . ($langs ? ' AND Locale NOT IN ("' . implode('","', $langs) . '")' : ''), false);
		if($allDels){
			$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND DID IN(' . implode($allDels) . ')');
			$allDels[$this->Language] = $this->ID;
			foreach($allDels as $lang => $did){
				$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND DID IN(' . implode(',', array_merge($langlinks, $allDels, [
						$this->ID])) . ') AND Locale="' . $lang . '" AND LDID=' . $did);
			}
		}
		if(!$langs){//all langs removed, we're done
			return true;
		}

		//check if any target lang has a different document in the language of this document
		if(($conflict = f('SELECT DID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND DID IN(' . implode(',', $langlinks) . ') AND Locale="' . $this->Language . '" AND LDID!=' . $this->ID . ' LIMIT 1'))){
			$this->errMsg = sprintf(g_l('weClass', '[languageLinksConflicts]'), array_search($conflict, $langlinks));

			return false;
		}

		//check target table lang entries
		$allTargets = $this->DB_WE->getAllFirstq('SELECT ID,Language FROM ' . $this->Table . ' WHERE ID IN(' . implode(',', $langlinks) . ')', false);
		$errs = [];
		//update lang entries
		foreach($langlinks as $lang => $did){
			if($allTargets[$did] != $lang){
				$errs[] = sprintf(g_l('weClass', '[languageLinksLangNotok]'), $lang, $allTargets[$did], $lang);
				//don't change/update this entry
				continue;
			}
			//forward link
			$this->DB_WE->query('REPLACE INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
					'DocumentTable' => $docTable,
					'DID' => $this->ID,
					'Locale' => $lang,
					'LDID' => $did
			]));

			if($withBackLink){
				//backward link
				$this->DB_WE->query('REPLACE INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
						'DocumentTable' => $docTable,
						'DID' => $did,
						'Locale' => $this->Language,
						'LDID' => $this->ID
				]));
			}
		}
		if($errs){
			$this->errMsg = implode("\n", $errs);
			t_e($errs);

			return false;
		}
		$this->setLanguageLinkBacklinks($docTable, $langlinks);
		//make sure to reread langlinks
		$this->getLangLinks();
		return true;
	}

	protected function setLanguageLinkBacklinks($docTable, array $langlinks){
		$allLinks = $this->DB_WE->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND LDID IN(' . implode(',', $langlinks) . ') GROUP BY Locale', false);
		foreach($langlinks as $lang => $did){
			foreach($allLinks as $locale => $ldid){
				if($lang === $locale){
					continue;
				}

				//backward link
				$this->DB_WE->query('INSERT IGNORE INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
						'DocumentTable' => $docTable,
						'DID' => $did,
						'Locale' => $locale,
						'LDID' => $ldid
				]));
			}
		}
	}

	public function setIgnoreLangLinkErr($ignore = false){
		$this->ignoreLangLinkErr = $ignore;
	}

	/**
	 * Before writing LangLinks to the db, we must check the Document-Locale: if it has changed, we must update or clear
	 * existing LangLinks from and to this document.
	 */
	protected function setLanguageLink(){
		$ret = true;
		$whatchanged = 0;
		if(!(LANGLINK_SUPPORT) || !($whatchanged = $this->checkLangLinksHaveChanged())){
			return true;
		}

		if($whatchanged | 1){//lang of doc has changed
			$this->saveLangLinkDocLang();
		}

		if($whatchanged | 2){//entries of docs have changed
			$ret = $this->saveLangLinkDocuments() && $ret;
		}

		return $this->ignoreLangLinkErr ? true : $ret;
	}

	/** returns error-messages recorded during an operation, currently only save is used */
	public function getErrMsg(){
		return ($this->errMsg ? '\n' . strtr($this->errMsg, ["\n" => '\n']) : '');
	}

	/** returns warning-messages recorded during an operation, currently only save is used */
	public function getWarningMsg(){
		return ($this->warningMsg ? '\n' . strtr(implode('\n\n', $this->warningMsg), ["\n" => '\n']) : '');
	}

	/** returns notice-messages recorded during an operation, currently only save is used */
	public function getNoticeMsg(){
		return ($this->noticeMsg ? '\n' . strtr(implode('\n\n', $this->noticeMsg), ["\n" => '\n']) : '');
	}

	public function getElementA($name, array $default = [], $defaultOnEmpty = false){
		return $this->getElement($name, self::ETYPE_TEXT, $default, $defaultOnEmpty); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function getElementS($name, $default = '', $defaultOnEmpty = false){
		return (string) $this->getElement($name, self::ETYPE_TEXT, $default, $defaultOnEmpty); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function getElementI($name, $default = 0, $defaultOnEmpty = false){
		$ret = $this->getElement($name, self::ETYPE_NUMBER, $default, $defaultOnEmpty); //@phan-suppress-current-line PhanDeprecatedFunction
		if(!$ret && isset($this->elements[$name][self::ETYPE_TEXT])){
			return intval($this->getElement($name, self::ETYPE_TEXT, $default, $defaultOnEmpty)); //@phan-suppress-current-line PhanDeprecatedFunction
		}
		return (int) $ret;
	}

	public function getElementB($name){
		$ret = $this->getElement($name, self::ETYPE_NUMBER, -1, true); //@phan-suppress-current-line PhanDeprecatedFunction
		if($ret === -1){
			if(isset($this->elements[$name][self::ETYPE_TEXT])){
				return boolval($this->getElement($name, self::ETYPE_TEXT, false)); //@phan-suppress-current-line PhanDeprecatedFunction
			}

			return false;
		}

		return boolval($ret);
	}

}
