<?php

/**
 * webEdition CMS
 *
 * $Rev: d51b7cd719c2 $
 * $Author: mokraemer $
 * $Date: 2019-07-04 15:22 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_doctype_frames extends we_modules_frame{
	const TAB_PROPERTIES = 1;

	public function __construct($frameset){
		parent::__construct($frameset);
		$this->module = we_modules_info::DOCTYPE;
		$this->Tree = new we_doctype_tree($this->jsCmd);
		$this->View = new we_doctype_view();
	}

	function getHTML($what = '', $mode = '', $step = 0){
		switch($what){
			case 'edfooter':
				return $this->getHTMLEditorFooter([
						'main' => [
							we_html_button::FOOTER_SAVE => [[we_base_perm::WORK_DOCTYPE_EDIT], 'save_docType'],
						],
						'optional' => [
							we_html_button::FOOTER_TRASH => [[we_base_perm::WORK_DOCTYPE_EDIT], ['confirmDeleteDocType', $this->View->Model->DocType, $this->View->Model->ID]]
						]
				]);

			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLEditorHeader($mode = 0){
		if(we_base_request::_(we_base_request::BOOL, "home")){
			return parent::getHTMLEditorHeader(0);
		}

		$we_tabs = new we_gui_tabs();

		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, self::TAB_PROPERTIES, ['id' => "tab_" . self::TAB_PROPERTIES, 'title' => g_l('modules_voting', '[property]')]);

		$this->jsCmd->addCmd('setTab', self::TAB_PROPERTIES);
		$tabsHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/doctype/doctype_top.js');


		$body = we_html_element::body([
				'onresize' => "weTabs.setFrameSize()",
				'onload' => "weTabs.setFrameSize();document.getElementById('tab_" . self::TAB_PROPERTIES . "').className='tabActive';",
				'id' => "eHeaderBody"
				], '<div id="main"><div id="headrow"><b>' . g_l('weClass', '[doctypes]') . ':&nbsp;</b><span id="h_path" class="header_small"><b id="titlePath">' . str_replace(" ", "&nbsp;", $this->View->Model->DocType) . '</b> (ID:' . $this->View->Model->ID . ')</span></div>' .
				$we_tabs->getHTML() .
				'</div>'
		);

		return $this->getHTMLDocument($body, $tabsHead);
	}

	protected function getHTMLEditorBody(){
		if(we_base_request::_(we_base_request::BOOL, "home")){
			return $this->View->getHomeScreen();
		}

		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody', 'onload' => "loaded=1;", 'onunload' => "doUnload()"], we_html_element::form([
						'name' => 'we_form', 'onsubmit' => "return false"], $this->View->getCommonHiddens(['cmd' => 'doctype_edit', 'pnt' => 'edbody']) . $this->getHTMLProperties($this->jsCmd))), $this->View->getJSProperty() . $this->jsCmd->getCmds());
	}

	function getHTMLTab1(we_base_jsCmd $jsCmd){
		return [
			['headline' => g_l('weClass', '[name]'),
				'html' => $this->View->Model->formName(),
				'icon' => we_html_multiIconBox::PROP_PATH,
			],
			['headline' => g_l('global', '[templates]'),
				'html' => $this->View->Model->formDocTypeTemplates($jsCmd),
				'icon' => we_html_multiIconBox::PROP_DOC,
			],
			['headline' => g_l('weClass', '[defaults]'),
				'html' => $this->View->Model->formDocTypeDefaults($jsCmd),
				'icon' => we_html_multiIconBox::PROP_ATTRIB,
			]
		];
	}

	function getHTMLProperties(we_base_jsCmd $jsCmd){// TODO: move to View
		//$t = we_base_request::_(we_base_request::INT, 'tabnr', 1);
		return we_html_element::jsScript(JS_DIR . 'utils/multi_editMulti.js') .
			we_html_element::div(['id' => 'tab1'], we_html_multiIconBox::getHTML('', $this->getHTMLTab1($jsCmd), true, '', -1, '', '', false));
	}

	protected function getHTMLCmd(){
		if(($pid = we_base_request::_(we_base_request::INT, "pid")) === false){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::_(we_base_request::INT, "offset", 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_doctype_tree::getItems($pid, $offset, $this->Tree->default_segment)]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(
						['name' => 'we_form'], we_html_element::hiddens([
							'pnt' => "cmd",
							'cmd' => "no_cmd"]
						)
					)
		));
	}

}
