<?php

/**
 * webEdition CMS
 *
 * $Rev$
 * $Author$
 * $Date$
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_glossary_frames extends we_modules_frame{

	public function __construct($frameset){
		parent::__construct($frameset);
		$this->module = we_modules_info::GLOSSARY;

		$this->Tree = new we_glossary_tree($this->jsCmd);
		$this->View = new we_glossary_view();
	}

	function getHTML($what = '', $mode = '', $step = 0){
		switch($what){
			case 'show_settings':

				return $this->getSettingsDialog();

			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	private function getSettingsDialog(){
		// Automatic Replacement
		$parts = [[
			'headline' => '',
			'html' => we_html_forms::checkboxWithHidden(we_glossary_replace::useAutomatic(), 'GlossaryAutomaticReplacement', g_l('modules_glossary', '[enable_replacement]')),
			'noline' => true]
		];

		$content = '<form name="we_form" target="cmdFrame" action="' . WEBEDITION_DIR . 'we_showMod.php?mod=glossary&pnt=show_settings">
	' . we_html_element::hidden('cmd', 'save_glossary_setting') . '
	' . we_html_multiIconBox::getHTML('GlossaryPreferences', $parts, true, we_html_button::position_yes_no_cancel(
					we_html_button::create_button(we_html_button::SAVE, 'javascript:document.we_form.submit();'), null, we_html_button::create_button(we_html_button::CLOSE, 'javascript:top.window.close();')), -1, '', '', false, g_l('modules_glossary', '[menu_settings]')) . '
	</form>';

		return
			we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(JS_DIR . 'formFunctions.js'), we_html_element::body(['class' => 'weDialogBody']
					, we_html_element::exIFrame('content', $content, 'position:absolute;top:0;bottom:1px;left:0;right:0;overflow: hidden;') .
					we_html_element::iframe('cmdFrame', 'about:blank', 'position:absolute;height:1px;bottom:0;left:0;right:0;overflow: hidden;')
		));
	}

	protected function getHTMLEditorHeader($mode = 0){
		if(we_base_request::_(we_base_request::BOOL, "home")){
			//FIXME: remove
			return parent::getHTMLEditorHeader(0);
		}
		$cmdid = we_base_request::_(we_base_request::STRING, 'cmdid');
		if($cmdid && !is_numeric($cmdid)){
			$this->View->Model->Language = substr($cmdid, 0, 5);
		}
		switch(we_base_request::_(we_base_request::STRING, 'cmd')){
			// Folder View
			case 'glossary_view_folder':
				return we_glossary_frameEditorFolder::Header($this);
			// Type View
			case 'glossary_view_type':
				return we_glossary_frameEditorType::Header($this);
			// Exception View
			case 'glossary_view_exception':
			case 'save_exception':
				return we_glossary_frameEditorException::Header($this);
			// Item View
			default:
				return we_glossary_frameEditorItem::Header($this);
		}
	}

	protected function getHTMLEditorBody(){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return $this->View->getHomeScreen();
		}
		$cmdid = we_base_request::_(we_base_request::STRING, 'cmdid');
		if($cmdid && !is_numeric($cmdid)){
			$this->View->Model->Language = substr($cmdid, 0, 5);
		}
		switch(we_base_request::_(we_base_request::STRING, 'cmd')){
			// Folder View
			case 'glossary_view_folder':
				return we_glossary_frameEditorFolder::Body($this);
			// Type View
			case 'glossary_view_type':
				return we_glossary_frameEditorType::Body($this);
			// Exception View
			case 'glossary_view_exception':
			case 'save_exception':
				return we_glossary_frameEditorException::Body($this);
			// Item View
			default:
				return we_glossary_frameEditorItem::Body($this);
		}
	}

	protected function getHTMLEditorFooter(array $btn_cmd = [], $extraHead = ''){
		if(we_base_request::_(we_base_request::BOOL, "home")){
			return parent::getHTMLEditorFooter([]);
		}
		$cmdid = we_base_request::_(we_base_request::STRING, 'cmdid');
		if($cmdid && !is_numeric($cmdid)){
			$this->View->Model->Language = substr($cmdid, 0, 5);
		}
		switch(we_base_request::_(we_base_request::STRING, 'cmd')){
			// Folder View
			case 'glossary_view_folder':
				return we_glossary_frameEditorFolder::Footer($this);
			// Type View
			case 'glossary_view_type':
				return we_glossary_frameEditorType::Footer($this);
			// Exception View
			case 'glossary_view_exception':
			case 'save_exception':
				return we_glossary_frameEditorException::Footer($this);
			// Item View
			default:
				return we_glossary_frameEditorItem::Footer($this);
		}
	}

	protected function getHTMLCmd(){
		if(($pid = we_base_request::_(we_base_request::RAW, "pid")) === false){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::_(we_base_request::INT, "offset", 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_glossary_tree::getItems($pid, $offset, $this->Tree->default_segment)]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(['name' => 'we_form'], we_html_element::hiddens(['pnt' => "cmd",
							'cmd' => "no_cmd"])
					)
				)
		);
	}

}
