<?php
/**
 * webEdition CMS
 *
 * $Rev: a2003b43d6bf $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-06-21 12:06 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class we_html_table
 *
 * Provides functions for creating html tags used in forms.
 */
class we_html_table extends we_html_baseCollection{
	private $cols = 0;

	/**
	 * Constructor of class we_html_table
	 *
	 * @param      $attribs                                array
	 * @param      $rows_num                               int                 (optional)
	 * @param      $cols_num                               int                 (optional)
	 *
	 */
	public function __construct(array $attribs = [], $rows_num = 0, $cols_num = 0, array $content = null){
		parent::__construct('table', true, $attribs);
		if($rows_num){
			$this->addRow($rows_num);
			$this->addCol($cols_num);
		} else {
			$this->cols = $cols_num;
		}
		$this->setTableContent($content);
	}

	public function setTableContent(array $content = null){
		if(!$content){
			return;
		}
		foreach($content as $rowNo => $rowContent){
			foreach($rowContent as $colNo => $col){
				if(is_array($col)){
					$this->setCol($rowNo, $colNo, ($col[0] && is_array($col[0]) ? $col[0] : []), $col[1]);
				} else {
					$this->setColContent($rowNo, $colNo, $col);
				}
			}
		}
	}

	/**
	 * This function adds a row to the table
	 *
	 * @param      $rows_num                               int                 (optional)
	 *
	 * @see        we_html_table()
	 *
	 * @return     integer
	 */
	public function addRow($rows_num = 1, array $attribs = []){
		$ret = count($this->childs);
		for($i = 0; $i < $rows_num; $i++){
			$tr = new we_html_baseCollection('tr');
			$tr->setAttributes($attribs);
			for($j = 0; $j < $this->cols; $j++){
				$tr->childs[] = new we_html_baseElement('td');
			}
			$this->childs[] = $tr;
		}
		return $ret;
	}

	/**
	 * This function adds a column to the table
	 *
	 * @param      $cols_num                               int                 (optional)
	 *
	 * @see        we_html_table()
	 *
	 * @return     void
	 */
	public function addCol($cols_num = 1){
		$this->cols += $cols_num;
		for($i = 0; $i < $cols_num; $i++){
			foreach($this->childs as &$v){
				$v->childs[] = new we_html_baseElement('td');
			}
		}
	}

	public function addHeaderRow(array $cols, array $rowAttr = []){
		$tr = new we_html_baseCollection('tr');
		$tr->setAttributes($rowAttr);
		for($j = 0; $j < $this->cols; $j++){
			$child = new we_html_baseElement('th');
			if(isset($cols[$j])){
				$child->content = $cols[$j];
			}
			$tr->childs[] = $child;
		}
		array_unshift($this->childs, $tr);
	}

	/**
	 * This functions sets the current row being edited
	 *
	 * @param      $rowid                                  int
	 * @param      $attribs                                array
	 * @param      $cols_num                               int                 (optional)
	 *
	 * @return     void
	 */
	/* public function setRow($rowid, array $attribs = [], $cols_num = 0){
	  $row = & $this->getChild($rowid);
	  $row->setAttributes($attribs);

	  if($cols_num){
	  if(empty($row->childs[$cols_num])){
	  $row->addChild(new we_html_baseElement('td'));
	  } else if($cols_num < count($row->childs)){
	  $row->childs = array_splice($row->childs, ($cols_num - 1));
	  }
	  }
	  } */

	/**
	 * This function sets the current column being edited
	 *
	 * @param      $rowid                                  int
	 * @param      $colid                                  int
	 * @param      $attribs                                array               (optional)
	 * @param      $content                                string              (optional)
	 *
	 * @return     void
	 */
	public function setCol($rowid, $colid, array $attribs = [], $content = ''){
		while(!isset($this->childs[$rowid])){
			$this->addRow();
		}
		$row = $this->getChild($rowid);
		$col = $row->getChild($colid);
		if(!$col){
			t_e('not enough cols defined (' . $this->cols . ')');
			while(!$col){
				$this->addCol();
				$col = $this->getChild($rowid)->getChild($colid);
			}
		}
		$col->setAttributes($attribs);
		if(($cspan = $col->getAttribute('colspan')) && --$cspan){
			for($pos = 1; $pos <= $cspan; $pos++){
				unset($row->childs[$colid + $pos]);
			}
		}
		$col->content = $content;
	}

	/**
	 * Assigns the attributes of a column
	 *
	 * @param      $rowid                                  int
	 * @param      $colid                                  int
	 * @param      $attribs                                array               (optional)
	 *
	 * @return     void
	 */
	public function setRowAttributes($rowid, array $attribs = []){
		$row = & $this->getChild($rowid);
		$row->setAttributes($attribs);
	}

	/**
	 * Assigns the attributes of a column
	 *
	 * @param      $rowid                                  int
	 * @param      $colid                                  int
	 * @param      $attribs                                array               (optional)
	 *
	 * @return     void
	 */
	public function setColAttributes($rowid, $colid, $attribs = []){
		$col = & $this->getChild($rowid)->getChild($colid);
		$col->setAttributes($attribs);
	}

	/**
	 * Sets the content of a column
	 *
	 * @param      $rowid                                  int
	 * @param      $colid                                  int
	 * @param      $content                                string              (optional)
	 *
	 * @return     void
	 */
	public function setColContent($rowid, $colid, $content = ''){
		while(!isset($this->childs[$rowid])){
			$this->addRow();
		}
		$col = $this->getChild($rowid)->getChild($colid);
		if($col === null){
			t_e('err');
		}
		$col->content = $content;
	}

}
