<?php

/**
 * webEdition CMS
 *
 * $Rev: 77af13e64c81 $
 * $Author: mokraemer $
 * $Date: 2019-07-15 10:49 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_html_tools{
	const OPTGROUP = '<!--we_optgroup-->';
	const TYPE_NONE = 0;
	const TYPE_ALERT = 1;
	const TYPE_INFO = 2;
	const TYPE_QUESTION = 3;
	const TYPE_HELP = 4; //shows a small icon with text as title
	const TYPE_LINK = 5;

	/** we_html_tools::protect()
	  protects a page. Guests can not see this page
	 * if redirect=true, return a code
	 */
	static function protect(array $perms = [], $redirect = ''){
		if(empty($_SESSION['user']['isWeSession']) || !isset($_SESSION['user']) || empty($_SESSION['user']['Username'])){
			self::setHttpCode(401);
			$allow = false;
			$hasWE = false;
		} else {
			if($perms){
				$allow = false;
				foreach($perms as $perm){
					$allow = $allow || we_base_permission::hasPerm($perm);
				}
				if(!$allow){
					self::setHttpCode(403);
					$hasWE = true;
				}
			} else {
				$allow = true;
			}
		}
		if(!$allow){
			if(session_status() === PHP_SESSION_ACTIVE){
				session_write_close();
			}
			if($redirect === false){
				return false;
			}
			echo self::getHtmlTop('', '', '', we_html_element::jsElement('alert("' . g_l('alert', ($hasWE ? '[no_perms_action]' : '[perms_no_permissions]')) . '");' . ($redirect ? 'document.location="' . $redirect . '"' : 'top.close();')), '<body>' .
				str_replace('\n', '<br/>', g_l('alert', ($hasWE ? '[no_perms_action]' : '[perms_no_permissions]'))) .
				'</body>'
			);
			exit();
		}
		if($redirect === false){
			return true;
		}
	}

	/**
	 * This function creates a table.
	 *
	 * @param          string                                  $element
	 * @param          string                                  $text
	 * @param          string                                  $textalign          (optional)
	 * @param          string                                  $textclass          (optional)
	 * @param          string                                  $col2               (optional)
	 * @param          string                                  $col3               (optional)
	 * @param          string                                  $col4               (optional)
	 * @param          string                                  $col5               (optional)
	 * @param          string                                  $col6               (optional)
	 * @param          int                                     $abstand            (optional)
	 *
	 * @return         string
	 */
	static function formElementTable($text){
		$elemOut = '';
		$args = func_get_args();
		array_shift($args);
		$colspan = count($args);
		foreach($args as $var){
			if(empty($var)){
				continue;
			}
			$tmp = '<td';
			if(is_array($var)){
				$colText = $var['text'];
				unset($var['text']);
				$tmp .= self::makeHTMLTagAtts($var);
			} else {
				$colText = $var;
			}
			$tmp .= '>' . $colText . '</td>';
			$elemOut .= $tmp;
		}
		return '<table class="default noSpace">' .
			($text ? '<tr><td colspan="' . $colspan . '">' . $text . '</td></tr>' : '') .
			'<tr>' . $elemOut . '</tr></table>';
	}

	public static function makeHTMLTagAtts(array $attribs){
		$ret = '';
		foreach($attribs as $k => $v){
			if(is_array($v)){
				t_e('array for html tag given', $k, $v);
				continue;
			}
			if(empty($k) || ($k !== 'value' && $v === '')){//keep empty value attribute (since option-tag without value attribute uses data as value
				continue;
			}
			$ret .= ' ' . ($k === 'link_attribute' ? // Bug #3741
				$v :
				str_replace('pass_', '', $k) . '="' . oldHtmlspecialchars($v, -1, 'ISO-8859-1', false) . '"');
		}
		return $ret;
	}

	public static function targetBox($name, $width = '', $id = '', $value = '', $onChange = '', $selectboxWidth = '', $disabled = false){
		$jsvarname = str_replace(['[', ']'], '_', $name);
		$inputs = ['class' => 'weSelect',
			'name' => 'sel_' . $name,
			'onfocus' => 'change' . $jsvarname . '=1;',
			'onchange' => "if(this.selectedIndex){
			this.form.elements['" . $name . "'].value = this.options[this.selectedIndex].text;
}
change" . $jsvarname . '=1;
this.selectedIndex=0;' .
			$onChange,
			'style' => (($selectboxWidth != '') ? ('width: ' . $selectboxWidth . 'px;') : ''),
		];

		if($disabled){
			$inputs['disabled'] = 'true';
		}

		$target_box = new we_html_select($inputs);
		$target_box->addOptions([
			'' => '',
			'_top' => '_top',
			'_parent' => '_parent',
			'_self' => '_self',
			'_blank' => '_blank'
		]);

		if($width){
			$inputs['style'] = 'width: ' . $width . 'px;';
		}

		if($id){
			$inputs['id'] = $id;
		}

		if($onChange){
			$inputs['onchange'] = $onChange;
		}

		return we_html_element::span(['class' => 'defaultfont', 'style' => 'margin-right:10px'], self::textInput($name, $value, '', $inputs, 'text', $width, 0, '', $disabled)) . $target_box->getHtml();
	}

	public static function textInput($name, $value = '', $maxlength = 0, array $attribs = [], $type = 'text', $width = 0, $height = 0, $markHot = '', $disabled = false, $readonly = false){
		if($markHot){
			$attribs['onchange'] = (isset($attribs['onchange']) ? $attribs['onchange'] : '') . "we_cmd('setHot');" . $markHot . ".hot=true;";
		}
		$attribs['class'] = 'wetextinput ' . (isset($attribs['class']) ? $attribs['class'] : '');
		$attribs['type'] = $type;
		$attribs['name'] = $name;
		$attribs['value'] = oldHtmlspecialchars($value);
		if($maxlength){
			$attribs['maxlength'] = $maxlength;
		}
		if($disabled){
			$attribs['disabled'] = 'disabled';
		}
		if($readonly){
			$attribs['readonly'] = 'readonly';
		}
		if($width){
			$attribs['width'] = $width;
		}
		if($height){
			$attribs['height'] = $height;
		}
		return we_html_element::input($attribs);
	}

	public static function messageBox($w, $h, $content, $headline = '', $buttons = ''){
		return '<div class="htmlMessageBox" style="width:' . $w . 'px;height:' . $h . 'px;">' .
			($headline ? '<h1 class="header">' . $headline . '</h1>' : '') .
			'<div>' . $content . '</div><div class="buttons">' . $buttons . '</div></div>';
	}

	public static function dialogLayout($content, $headline, $buttons = '', $margin = true){
		return we_html_multiIconBox::getHTML('', [['html' => $content, 'headline' => '']], $margin, ($buttons ? '<div class="htmlDialogLayoutButtons">' . $buttons . '</div>' : ''), -1, '', '', false, $headline);
	}

	public static function dialogBorder($w, array $content, array $headline, $class = 'middlefont', $id = ''){
		$out = '<table' . ($id ? ' id="' . $id . '"' : '') . ($w ? ' style="width:' . $w . 'px;"' : '') . ' class="default">
		<tr class="boxHeader">';
		// HEADLINE
		foreach($headline as $h){
			$out .= '<td class="' . $class . '">' . $h . '</td>';
		}
		$out .= '</tr>';

		//CONTENT
		foreach($content as $c){
			$out .= '<tr class="htmlDialogBorder4Cell">' . self::dialogBorderRow($c, $class) . '</tr>';
		}
		$out .= '</table>';

		return $out;
	}

	private static function dialogBorderRow(array $content, $class){
		$out = '';

		foreach($content as $cur){
			$bgcolor = (!empty($cur['bgcolor']) ) ? $cur['bgcolor'] : '';
			$out .= '<td class="' . $class . '" style="' . ($bgcolor ? 'background-color:' . $bgcolor . '; ' : '') .
				(isset($cur['align']) ? 'text-align:' . $cur['align'] . ';' : '') . ' ' .
				(isset($cur['height']) ? 'height:' . $cur['height'] . 'px;' : '') . '">' .
				(!empty($cur['dat']) ? $cur['dat'] : "&nbsp;") .
				'</td>';
		}

		return $out;
	}

	public static function select($name, array $values, $size = 1, $selectedIndex = '', $multiple = false, array $attribs = [], $compareKey = true, $width = 0, $cls = '', $oldHtmlspecialchars = true){
		$ret = '';
		$selIndex = is_array($selectedIndex) ? $selectedIndex : ($selectedIndex ? explode(',', $selectedIndex) : ['']);
		$optgroup = $selFound = false;
		foreach($values as $value => $text){
			if($text === self::OPTGROUP || $value === self::OPTGROUP){
				$ret .= ($optgroup ? '</optgroup>' : '') . '<optgroup label="' . ($oldHtmlspecialchars ? oldHtmlspecialchars($value) : $value) . '">';
				$optgroup = true;
				continue;
			}
			$class = (is_array($cls) && !empty($cls[$value]) ? $cls[$value] : '');
			//setting compare to true will fail for e.g. template selection in document
			$isSel = !$selFound && in_array(($compareKey ? $value : $text), $selIndex);
			$selFound = $selFound || (!$multiple && $isSel);
			$ret .= '<option value="' . ($oldHtmlspecialchars ? oldHtmlspecialchars($value) : $value) . '"' .
				($class ? ' class="' . $class . '"' : '') .
				($isSel ? ' selected="selected"' : '') . '>' .
				($oldHtmlspecialchars ? oldHtmlspecialchars($text) : $text) . '</option>';
		}
		$ret .= ($optgroup ? '</optgroup>' : '');
		return ($name ? we_html_element::select(array_merge([
				'class' => 'weSelect ' . (is_array($cls) ? '' : $cls),
				'name' => trim($name),
				'size' => abs($size),
				($multiple ? 'multiple' : '') => 'multiple',
				($width ? 'width' : '') => ($width ?: '')
					], $attribs
				), $ret) : $ret);
	}

	//FIXME: make fn more concise and make base all country selects on it
	public static function selectCountry($name = '', $id = '', $size = 1, $selected = [], $multiple = false, array $attribs = [], $width = 50, $cls = 'defaultfont', $oldHtmlspecialchars = true, $optsOnly = false){
		$langcode = array_search($GLOBALS['WE_LANGUAGE'], getWELangs());
		$countrycode = array_search($langcode, getWECountries());

		$attributes = ['name' => $name,
			'id' => ($id ?: $name),
			'size' => $size,
			'width' => $width,
			'style' => (isset($attribs['style']) ? $attribs['style'] : ''),
			'class' => 'weSelect ' . $cls
		];
		if($multiple){
			$attributes['multiple'] = 'multiple';
		}
		$countryselect = new we_html_select($attributes);

		$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);

		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		setlocale(LC_ALL, $oldLocale);

		if(WE_COUNTRIES_DEFAULT != ''){
			$countryselect->addOption('--', ($oldHtmlspecialchars ? oldHtmlspecialchars(CheckAndConvertISObackend(WE_COUNTRIES_DEFAULT)) : CheckAndConvertISObackend(WE_COUNTRIES_DEFAULT)));
		}
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryselect->addOption($countrykey, ($oldHtmlspecialchars ? oldHtmlspecialchars(CheckAndConvertISObackend($countryvalue)) : CheckAndConvertISObackend($countryvalue)));
		}
		unset($countryvalue);
		if(!empty($topCountries) && !empty($shownCountries)){
			$countryselect->addOption('-', '----', ['disabled' => 'disabled']);
		}
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryselect->addOption($countrykey, ($oldHtmlspecialchars ? oldHtmlspecialchars(CheckAndConvertISObackend($countryvalue)) : CheckAndConvertISObackend($countryvalue)));
		}
		unset($countryvalue);

		foreach($selected as $val){
			$countryselect->selectOption($val);
			if(!$multiple){
				break;
			}
		}

		return $optsOnly ? $countryselect->getOptionsArray() : $countryselect->getHtml();
	}

	public static function inputChoiceField($name, $value, array $values, $atts, $mode, $valuesIsHash = false){
		//  This function replaced we_getChoiceField
		//  we need input="text" and select-box
		//  First input='text'
		$textField = getHtmlTag('input', array_merge($atts, ['type' => 'text', 'name' => $name, 'value' => oldHtmlspecialchars($value)]));

		$opts = getHtmlTag('option', ['value' => ''], '', true);
		$attsOpts = [];

		if($valuesIsHash){
			foreach($values as $val => $text){
				$attsOpts['value'] = oldHtmlspecialchars($val);
				$opts .= getHtmlTag('option', $attsOpts, oldHtmlspecialchars($text));
			}
		} else {
			// options of select Menu
			if(isset($atts['xml'])){
				$attsOpts['xml'] = $atts['xml'];
			}

			foreach($values as $option){
				$attsOpts['value'] = oldHtmlspecialchars($option);
				$opts .= getHtmlTag('option', $attsOpts, oldHtmlspecialchars($option));
			}
		}

		// select menu

		if(isset($atts['id'])){ //  use another ID!!!!
			$atts['id'] = 'tmp_' . $atts['id'];
		}
		$atts['onchange'] = 'this.form.elements[\'' . $name . '\'].value' . ($mode === 'add' ?
			' += ((this.form.elements[\'' . $name . '\'].value ? \' \' : \'\') + this.options[this.selectedIndex].value);' :
			'=this.options[this.selectedIndex].value;'
			) . 'this.selectedIndex=0;';
		$atts['name'] = 'tmp_' . $name;
		//$atts['size'] = isset($atts['size']) ? $atts['size'] : 1;
		$selectMenue = getHtmlTag('select', removeAttribs($atts, ['size']), $opts, true); //  remove size for choice
		return '<table class="default"><tr><td>' . $textField . '</td><td>' . $selectMenue . '</td></tr></table>';
	}

	public static function getExtensionPopup($name, $selected, $extensions, $width = '', array $attribs = [], $permission = true){
		if((isset($extensions)) && (count($extensions) > 1)){
			if(!$permission){
				$disabled = ' disabled="disabled "';
				$attribs['disabled'] = 'disabled';
			} else {
				$disabled = '';
			}
			$out = '<table class="default"><tr><td>' .
				self::textInput($name, $selected, '', $attribs, "text", $width / 2, 0, "top") .
				'</td><td><select class="weSelect" name="wetmp_' . $name . '" ' . $disabled . ($width ? ' style="width: ' . ($width / 2) . 'px"' : '') . ' onchange="we_cmd(\'setHot\');if(this.options[this.selectedIndex].text){this.form.elements[\'' . $name . '\'].value=this.options[this.selectedIndex].text;}this.selectedIndex=0"><option>';
			foreach($extensions as $extension){
				$out .= '<option>' . $extension . '</option>';
			}
			$out .= '</select></td></tr></table>';
			return $out;
		}
		$ext = $extensions[0];
		return self::hidden($name, $ext) . '<b class="defaultfont">' . $ext . '</b>';
	}

	private static function we_getDayPos($format){
		return max([self::findChar($format, 'd'), self::findChar($format, 'D'), self::findChar($format, 'j')]);
	}

	private static function we_getMonthPos($format){
		return max([self::findChar($format, 'm'), self::findChar($format, 'M'), self::findChar($format, 'n'), self::findChar($format, 'F')]);
	}

	private static function we_getYearPos($format){
		return max([self::findChar($format, 'y'), self::findChar($format, 'Y')]);
	}

	private static function we_getHourPos($format){
		return max([self::findChar($format, 'g'), self::findChar($format, 'G'), self::findChar($format, 'h'), self::findChar($format, 'H')]);
	}

	private static function we_getMinutePos($format){
		return self::findChar($format, 'i');
	}

	private static function findChar($in, $searchChar){
		$pos = 0;
		while(($pos = strpos($in, $searchChar, $pos)) !== false){
			if(substr($in, $pos - 1, 1) != '\\'){
				return $pos;
			}
			++$pos;
		}
		return -1;
	}

	//note it is possible to set "no date"
	public static function getDateInput($name, $time = 0, $setHot = false, $format = '', $onchange = '', $class = 'weSelect', $xml = false, $minyear = 0, $maxyear = 0, $style = '', $langcode = '', $tableStyle = ''){
		$attsSelect = $attsOption = $attsHidden = $xml ? ['xml' => $xml] : [];

		$langcode = $langcode ?: array_search($GLOBALS['WE_LANGUAGE'], getWELangs());

		if($class){
			$attsSelect['class'] = $class;
		}
		if($style){
			$attsSelect['style'] = $style;
		}
		$attsSelect['size'] = 1;

		if($onchange || $setHot){
			$attsSelect['onchange'] = (($setHot ? "we_cmd('setHot');" : '') . $onchange);
		}

		if(is_object($time)){
			$day = $time->format('j');
			$month = $time->format('n');
			$year = $time->format('Y');
			$hour = $time->format('G');
			$minute = $time->format('i');
		} else {
			$time = $time > 2 ? $time : 0;
			$day = intval(date('j', $time));
			$month = intval(date('n', $time));
			$year = intval(date('Y', $time));
			$hour = intval(date('G', $time));
			$minute = intval(date('i', $time));
		}

		$dayPos = self::we_getDayPos($format);
		$monthPos = self::we_getMonthPos($format);
		$yearPos = self::we_getYearPos($format);
		$hourPos = self::we_getHourPos($format);
		$minutePos = self::we_getMinutePos($format);

		$showHour = true;
		$showMinute = true;

		$name = preg_replace('/^(.+)]$/', '${1}%s]', $name);
		if(!$format || $dayPos > -1){
			$days = getHtmlTag('option', array_merge($attsOption, ['value' => 0]), '--');

			for($i = 1; $i <= 31; $i++){
				$atts2 = ($time && $day == $i) ? ['selected' => 'selected'] : [];
				$days .= getHtmlTag('option', array_merge($attsOption, $atts2), sprintf('%02d', $i));
			}
			$daySelect = getHtmlTag('select', array_merge($attsSelect, ['name' => sprintf($name, '_day'),
					'id' => sprintf($name, '_day')
					]), $days, true) . '&nbsp;';
		} else {
			$daySelect = getHtmlTag('input', array_merge($attsHidden, ['type' => 'hidden',
				'name' => sprintf($name, '_day'),
				'id' => sprintf($name, '_day'),
				'value' => $time ? $day : 1
			]));
		}

		if(!$format || $monthPos > -1){
			$months = getHtmlTag('option', array_merge($attsOption, ['value' => 0]), '--');

			$monthType = (strpos($format, 'F') ? 'F' : (strpos($format, 'M') ? 'M' : '0'));
			$monthsArray = $monthType ? we_base_country::getTranslationList(we_base_country::MONTH, $langcode) : [];
			for($i = 1; $i <= 12; $i++){
				switch($monthType){//Bug #4095
					case 'F':
						$val = $monthsArray['wide'][$i];
						break;
					case 'M':
						$val = $monthsArray['abbreviated'][$i];
						break;
					default:
						$val = sprintf('%02d', $i);
				}

				$atts2 = ($time && $month == $i) ? ['selected' => 'selected', 'value' => $i] : ['value' => $i];
				$months .= getHtmlTag('option', array_merge($attsOption, $atts2), $val);
			}
			$monthSelect = getHtmlTag('select', array_merge($attsSelect, ['name' => sprintf($name, '_month'),
					'id' => sprintf($name, '_month')
					]), $months, true) . '&nbsp;';
		} else {
			$monthSelect = getHtmlTag('input', array_merge($attsHidden, ['type' => 'hidden',
				'name' => sprintf($name, '_month'),
				'id' => sprintf($name, '_month'),
				'value' => $time ? $month : 3 //we set it to march since in march we have 31 days & 1.1.1970 - 00:00 is undefined
			]));
		}
		if(!$format || $yearPos > -1){
			$years = getHtmlTag('option', array_merge($attsOption, ['value' => 0]), '----');
			$minyear = $minyear ?: 1970;
			$maxyear = $maxyear ?: abs(date('Y') + 100);

			for($i = $minyear; $i <= $maxyear; $i++){
				$atts2 = ($time && $year == $i) ? ['selected' => 'selected'] : [];
				$years .= getHtmlTag('option', array_merge($attsOption, $atts2), sprintf('%02d', $i));
			}
			$yearSelect = getHtmlTag('select', array_merge($attsSelect, ['name' => sprintf($name, '_year'),
					'id' => sprintf($name, '_year')
					]), $years, true) . '&nbsp;';
		} else {
			$yearSelect = getHtmlTag('input', array_merge($attsHidden, ['type' => 'hidden',
				'name' => sprintf($name, '_year'),
				'id' => sprintf($name, '_year'),
				'value' => $time ? $year : 0
			]));
		}

		if(!$format || $hourPos > -1){
			$hours = '';
			for($i = 0; $i <= 23; $i++){
				$atts2 = ($time && $hour == $i) ? ['selected' => 'selected'] : [];
				$hours .= getHtmlTag('option', array_merge($attsOption, $atts2), sprintf('%02d', $i));
			}
			$hourSelect = getHtmlTag('select', array_merge($attsSelect, ['name' => sprintf($name, '_hour'),
					'id' => sprintf($name, '_hour')
					]), $hours, true) . '&nbsp;';
		} else {
			$hourSelect = getHtmlTag('input', array_merge($attsHidden, ['type' => 'hidden',
				'name' => sprintf($name, '_hour'),
				'id' => sprintf($name, '_hour'),
				'value' => $time ? $hour : 0
			]));
			$showHour = false;
		}

		if(!$format || $minutePos > -1){
			$minutes = '';
			for($i = 0; $i <= 59; $i++){
				$atts2 = ($time && $minute == $i) ? ['selected' => 'selected'] : [];
				$minutes .= getHtmlTag('option', array_merge($attsOption, $atts2), sprintf('%02d', $i));
			}
			$minSelect = getHtmlTag('select', array_merge($attsSelect, ['name' => sprintf($name, '_minute'),
					'id' => sprintf($name, '_minute')
					]), $minutes, true) . '&nbsp;';
		} else {
			$minSelect = getHtmlTag('input', array_merge($attsHidden, ['type' => 'hidden',
				'name' => sprintf($name, '_minute'),
				'id' => sprintf($name, '_minute'),
				'value' => $time ? $minute : 0,
			]));
			$showMinute = false;
		}

		$datePosArray = [($dayPos == -1) ? 'd' : $dayPos => $daySelect,
			($monthPos == -1) ? 'm' : $monthPos => $monthSelect,
			($yearPos == -1) ? 'y' : $yearPos => $yearSelect
		];

		$timePosArray = [($hourPos == -1) ? 'h' : $hourPos => $hourSelect,
			($minutePos == -1) ? 'i' : $minutePos => $minSelect
		];

		ksort($datePosArray);
		ksort($timePosArray);

		return '<table id="' . sprintf($name, '_dateTable') . '" class="we we_date default" ' . ($tableStyle ? 'style="' . $tableStyle . '"' : '') . '><tr><td>' .
			implode('', $datePosArray) .
			($showHour || $showMinute ? '</td></tr><tr><td>' : '') .
			implode('', $timePosArray) .
			'</td></tr></table>';
	}

	public static function getHtmlTop($title = '', $charset = '', $doctype = '', $extraHead = '', $body = '', $skipErrorHandler = true){
		return we_html_element::docType($doctype) .
			we_html_element::html(we_html_element::head(self::getHtmlInnerHead($title ?: 'webEdition', $charset, $skipErrorHandler) . $extraHead, ($extraHead || $body ? true : false)) .
				$body, ($body ? true : false)
		);
	}

	public static function getJSErrorHandler($register = false){
		return we_html_element::jsScript(JS_DIR . 'utils/jsErrorHandler.js', ($register ? 'window.onerror=errorHandler;' : ''));
	}

	private static function getHtmlInnerHead($title, $charset, $skipErrorHandler){
		self::headerCtCharset('text/html', ($charset ?: $GLOBALS['we']['Charset']['Backend']));
		return
			//load this as early as possible
			($skipErrorHandler ?
			'' :
			self::getJSErrorHandler(true)
			) .
			we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
			self::metaCtCharset(($charset ?: $GLOBALS['we']['Charset']['Backend'])) .
			we_html_element::title($_SERVER['SERVER_NAME'] . ' ' . $title) .
			we_html_element::meta(['name' => 'viewport', 'content' => 'width=device-width, height=device-height, maximum-scale=1.0, initial-scale=1.0, user-scalable=yes']) .
			we_html_element::meta(['http-equiv' => 'expires', 'content' => 0]) .
			we_html_element::meta(['http-equiv' => 'Cache-Control', 'content' => 'no-cache']) .
			we_html_element::meta(['http-equiv' => 'pragma', 'content' => 'no-cache']) .
			we_html_element::meta(['http-equiv' => 'imagetoolbar', 'content' => 'no']) .
			we_html_element::meta(['name' => 'generator', 'content' => 'webEdition']) .
			we_html_element::linkElement(['rel' => 'SHORTCUT ICON', 'href' => IMAGE_DIR . 'webedition.ico']) .
			STYLESHEET .
			JQUERY;
	}

	public static function metaCtCharset($charset){
		$GLOBALS['we']['Charset']['Page'] = $charset;
		return we_html_element::meta(['charset' => $charset]);
	}

	public static function headerCtCharset($content, $charset, $skipsent = false){
		$GLOBALS['we']['Charset']['Page'] = $charset;
		if(!$skipsent || ($skipsent && !headers_sent())){
			header('Content-Type: ' . $content . '; charset=' . $charset, true);
			header('X-XSS-Protection: 0');
		}
	}

	public static function forceHeaders($charset){
		if(headers_sent() || !empty($GLOBALS['we']['Scheduler_active']) || !empty($GLOBALS['we']['Versioning_active'])){
			return;
		}
		self::headerCtCharset('text/html', $charset);
		self::setHttpCode(200);
		//remove possible redirect

		header_remove('Location');
	}

	/**
	 * @param string $text
	 * @param string $img
	 * @param bool $yes
	 * @param bool $no
	 * @param bool $cancel
	 * @param string $yesHandler
	 * @param string $noHandler
	 * @param string $cancelHandler
	 * @param string $script
	 * @return string
	 */
	public static function yesNoCancelDialog($text = '', $img = '', $yes = '', $no = '', $cancel = '', $yesHandler = '', $noHandler = '', $cancelHandler = ''){
		$cancelButton = ($cancel ? we_html_button::create_button(we_html_button::CANCEL, 'javascript:' . $cancelHandler) : '');
		$noButton = ($no ? we_html_button::create_button(we_html_button::NO, 'javascript:' . $noHandler) : '');
		$yesButton = ($yes ? we_html_button::create_button(we_html_button::YES, 'javascript:' . $yesHandler) : '');


		$content = new we_html_table(['class' => 'default'], 1, ($img ? 2 : 1));

		if($img){
			$content->setCol(0, 0, ['style' => 'vertical-align:top;padding:10px;'], $img);
		}

		$content->setCol(0, ($img ? 1 : 0), ['class' => 'defaultfont', 'style' => 'padding:10px;'], $text);

		return self::dialogLayout($content->getHtml(), '', we_html_button::position_yes_no_cancel($yesButton, $noButton, $cancelButton));
	}

	public static function groupArray(array $arr, $sort = true, $len = 1){
		$tmp = [];
		if($sort){
			asort($arr, SORT_STRING);
		}
		$pre = '';
		foreach($arr as $key => $value){
			$newPre = strtoupper(substr($value, 0, $len));
			if($pre != $newPre){
				//we add an extra space so it never interferes with numeric keys
				$tmp[' ' . $newPre] = self::OPTGROUP;
				$pre = $newPre;
			}
			$tmp[$key] = $value;
		}
		return $tmp;
	}

	/* displays a grey box with text and an icon

	  $text: Text to display
	  $type: 0=no icon
	  1=Alert icon
	  2=Info icon
	  3=Question icon
	  $width: width of box
	  $useHtmlSpecialChars: true or false
	 */

	public static function alertAttentionBox($text, $type = self::TYPE_NONE, $width = 0, $useHtmlSpecialChars = true, $clip = 0){
		switch($type){
			case self::TYPE_LINK:
				//we have link & text attached in an array
				list($link, $text) = $text;
			case self::TYPE_HELP:
				$width = false;
		}

		if($width === false){
			$class = 'infobox';
			$title = '<span>' . $text . '</span>';
		} else {
			$title = $class = '';
		}
		switch($type){
			case self::TYPE_ALERT:
				$icon = '<span class="fa-stack fa-lg alertIcon ' . $class . '"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i>' . $title . '</span>';
				break;
			case self::TYPE_INFO:
				$icon = '<span class="fa-stack fa-lg alertIcon ' . $class . '" style="color:#007de3;"><i class="fa fa-circle fa-stack-2x" ></i><i class="fa fa-info fa-stack-1x fa-inverse"></i>' . $title . '</span>';
				break;
			case self::TYPE_QUESTION:
				$icon = '<span class="fa-stack fa-lg alertIcon ' . $class . '"><i class="fa fa-circle fa-stack-2x" ></i><i style="color:black" class="fa fa-question fa-stack-1x"></i>' . $title . '</span>';
				break;
			case self::TYPE_HELP:
				return '<span class="fa-stack alertIcon ' . $class . '" style="color:inherit;"><i class="fa fa-question-circle" ></i>' . $title . '</span>';
			case self::TYPE_LINK:
				return '<span class="fa-stack alertIcon ' . $class . '" style="color:inherit;"><i class="fa fa-external-link-square" ></i>' . $title . '</span>';
			default:
				$icon = '';
		}
		if($width === false){
			return $icon;
		}

		$text = ($useHtmlSpecialChars) ? oldHtmlspecialchars($text, ENT_COMPAT, 'ISO-8859-1', false) : $text;

		if($clip){
			$unique = md5(uniqid(__FUNCTION__, true)); // #6590, changed from: uniqid(microtime())
		}

		return '<div class="alertAttentionBox' . ($icon ? ' alertIcon' : '') . ($clip ? ' alertCut' : '') . '" style="' . ($width ? ' width:' . $width . (is_numeric($width) ? 'px' : '') . ';' : '') . '">' .
			($icon ? '<div class="icon">' . $icon . '</div>' : '') .
			'<div class="middlefont ' . ($clip > 0 ? 'cutText" id="td_' . $unique . '" style="max-width:100%;"' : '"') . '>' . $text . '</div>' .
			($clip > 0 ? '<button type="button" class="weBtn clipbutton" id="btn_' . $unique . '" onclick="WE().util.clip(document,\'' . $unique . '\',' . $clip . ')"><i class="fa fa-lg fa-caret-right"></i></button>' : '') .
			'</div>';
	}

	public static function setHttpCode($status){
		switch($status){
			case 200:
				header('HTTP/1.0 200 OK', false, 200);
				header('Status: 200 OK', false, 200);
				break;
			case 206:
				header('HTTP/1.1 ' . $status . ' Partial Content', true, $status);
				header('Status: ' . $status . ' Partial Content', true, $status);
				break;
			case 303:
				header('HTTP/1.1 ' . $status . ' See Other', true, $status);
				header('Status: ' . $status . ' See Other', true, $status);
				break;
			case 304:
				header('HTTP/1.1 ' . $status . ' Not Modified', true, $status);
				header('Status: ' . $status . ' Not Modified', true, $status);
			case 307:
				header('HTTP/1.1 ' . $status . ' Temporary Redirect', true, $status);
				header('Status: ' . $status . ' Temporary Redirect', true, $status);
				break;
			case 400:
				header('HTTP/1.1 ' . $status . ' Bad Request', true, $status);
				header('Status: ' . $status . ' Bad Request', true, $status);
				break;
			case 401:
				header('HTTP/1.1 ' . $status . ' Unauthorized', true, $status);
				header('Status: ' . $status . ' Unauthorized', true, $status);
				break;
			case 403:
				header('HTTP/1.1 ' . $status . ' Forbidden', true, $status);
				header('Status: ' . $status . ' Forbidden', true, $status);
				break;
			case 404:
				header('HTTP/1.1 ' . $status . ' Not Found', true, $status);
				header('Status: ' . $status . ' Not Found', true, $status);
				break;
			case 408:
				header('HTTP/1.1 ' . $status . ' Request Time-out', true, $status);
				header('Status: ' . $status . ' Request Time-out', true, $status);
				break;
			case 416:
				header('HTTP/1.1 ' . $status . ' Requested Range Not Satisfiable', true, $status);
				header('Status: ' . $status . ' Requested Range Not Satisfiable', true, $status);
				break;
			case 503:
				header('HTTP/1.1 ' . $status . ' Service Unavailable', true, $status);
				header('Status: ' . $status . ' Service Unavailable', true, $status);
				break;
		}
	}

	/**
	 * @abstract get code for calendar
	 * @return html-code for calendar
	 */
	public static function getDateSelector($name, $btn, $value, $selWidth = 100, $btnClass = ''){
		static $cnt = 0;
		return we_html_element::span(['class' => 'default', 'id' => $name . '_cell'], we_html_tools::textInput($name, $value, 10, ['id' => 'selectorDate' . $cnt, 'class' => "wetextinput datepicker",
					'readonly' => "readonly"], "text", $selWidth) .
				we_html_button::create_button(we_html_button::CALENDAR, "javascript:$('#selectorDate" . $cnt++ . "').datepicker('show');", '', '', false, false, $btn, false, '', $btnClass)
		);
	}

}
