<?php
/**
 * webEdition CMS
 *
 * $Rev: 096b1a1b079c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-08 17:38 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_listview_multiobject
 * @desc    class for tag <we:listview type="multiobject">
 *
 */
class we_listview_multiobject extends we_listview_objectBase{
	private $objects = []; /* Array of all objetcs to show in this listview */

	/**
	 * @desc    constructor of class
	 *
	 * @param   name          string - name of listview
	 * @param   rows          integer - number of rows to display per page
	 * @param   offset        integer - start offset of first page
	 * @param   order         string - field name(s) to order by
	 * @param   desc          boolean - set to true, if order should be descendend
	 * @param   cats          string - comma separated categories
	 * @param   catOr         boolean - set to true if it should be an "OR condition"
	 * @param   condition     string - condition string (like SQL)
	 * @param   triggerID     integer - ID of a document which to use for displaying the detail page
	 * @param   cols          ??
	 * @param   seeMode       boolean - value if objects shall be accessible in seeMode (default true)
	 * @param   searchable 	  boolean - if false then show also documents which are not marked as searchable
	 * @param	unknown_type  $calendar
	 * @param	unknown_type  $datefield
	 * @param	unknown_type  $date
	 * @param	unknown_type  $weekstart
	 * @param	string        $categoryids
	 *
	 */
	public function __construct($name, $rows = 9999999, $offset = 0, $order = '', $desc = false, $cats = '', $catOr = '', $condition = '', $triggerID = '', $cols = '', $seeMode = true, $searchable = true, $calendar = '', $datefield = '', $date = '', $weekstart = '', $categoryids = '', $customerFilterType = 'false', $docID = 0, $languages = '', $hidedirindex = false, $objectseourls = false){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, [], $cols, $calendar, $datefield, $date, $weekstart, $categoryids, $customerFilterType, 0, $triggerID, $seeMode, $searchable, $docID, $languages, $objectseourls, $hidedirindex);

		$this->condition = $condition;

		$data = 0;
		$found = false;
		//check if current lv is of we_listview_object/we_listview_multiobject
		if(we_tag_listview::$current && we_tag_listview::$current instanceof we_listview_objectBase){
			if(($dat = we_tag_listview::$current->f($name))){
				$data = we_unserialize($dat);
			}
			$found = true;
		} elseif(!empty(we_tag_listview::$lvArray)){
			//find last we_listview_object in stack
			foreach(array_reverse(we_tag_listview::$lvArray) as $cur){
				if($cur instanceof we_listview_objectBase){
					if(($dat = $cur->f($name))){
						$data = we_unserialize($dat);
					}
					$found = true;
					break;
				}
			}
		}

		if(!$found && $GLOBALS['we_doc']->getElement($name)){
			$data = we_unserialize($GLOBALS['we_doc']->getElement($name));
		}

		$objects = (isset($data['objects']) ? $data['objects'] : $data);
		if(!is_array($objects)){
			return;
		}
		$objects = array_filter(array_map('intval', $objects));
		if(empty($objects)){
			return;
		}

		$this->objects = $objects;
		//FIXME: can we use defaultsArray?!
		$this->classID = f('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID IN (' . implode(',', $objects) . ') LIMIT 1');
		if(!$this->classID){
			return;
		}

		if($this->desc && (!preg_match('|.+ desc$|i', $this->order))){
			$this->order .= ' DESC';
		}

		// IMPORTANT for seeMode !!!! #5317
		$this->LastDocPath = (isset($_SESSION['weS']['last_webEdition_document']) ? $_SESSION['weS']['last_webEdition_document']['Path'] : '');

		$this->fillMatrix($this->classID);
		$calendar_select = '';
		$calendar_where = '';

		if($calendar){
			$this->fetchCalendar($this->condition, $calendar_select, $calendar_where, $this->matrix);
		}
		$sqlParts = $this->makeSQLParts($this->classID, $this->order, $this->condition);

		$weDocumentCustomerFilter_tail = (we_modules_info::isActive(we_modules_info::CUSTOMER) ? we_customer_documentFilter::getConditionForListviewQuery($this->customerFilterType, $this, $this->classID) : '');

		if($sqlParts['tables']){
			$where = implode(' AND ', array_filter([
					'ids' => '`of`.ID IN (' . implode(',', $this->objects) . ')',
					'search' => ($this->searchable ? '`of`.IsSearchable=1' : ''),
					'lang' => ($this->languages ? '`of`.Language IN ("' . implode('","', array_map('escape_sql_query', array_filter(array_map('trim', explode(',', $this->languages))))) . '")' : ''),
					'cat' => ($this->cats || $this->categoryids ? we_category_category::getCatSQLTail($this->cats, 'of', $this->catOr, $this->DB_WE, 'Category', $this->categoryids) : ''),
					'publ' => $sqlParts['publ_cond'],
					'cond' => ($sqlParts['cond'] ? '(' . $sqlParts['cond'] . ')' : ''),
					'cal' => $calendar_where,
				])) . $weDocumentCustomerFilter_tail . $sqlParts['groupBy'];

			$this->anz_all = f('SELECT COUNT(1) FROM ' . $sqlParts['tables'] . ' JOIN ' . OBJECT_FILES_TABLE . ' `of` ON `of`.ID=ob' . $this->classID . '.OF_ID WHERE ' . $where, '', $this->DB_WE);

			$this->DB_WE->query('SELECT ' . $sqlParts['fields'] . $calendar_select . ' FROM ' . $sqlParts['tables'] . ' JOIN ' . OBJECT_FILES_TABLE . ' `of` ON `of`.ID=ob' . $this->classID . '.OF_ID WHERE ' . $where . ($this->order ? $sqlParts['order'] : ' ORDER BY FIND_IN_SET(`of`.ID,"' . implode(',', $this->objects) . '")') . ($rows ? (' LIMIT ' . $this->start . ',' . $this->rows) : ''));

			$this->anz = $this->DB_WE->num_rows();
		} else {
			$this->anz_all = 0;
			$this->anz = 0;
		}
		if($calendar != ''){
			$this->postFetchCalendar();
		}
	}

}
