<?php

/**
 * webEdition CMS
 *
 * $Rev: 096b1a1b079c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-08 17:38 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class
 * @desc    class for tag <we:listview type='search'>
 *          the difference to the normal listview is, that you can only
 *          display the fields from the index table (tblIndex) which are
 *          Title, Description we_text, we_path
 *
 */
class we_listview_search extends we_listview_base{

	var $docType = ''; /* doctype string */
	var $class = 0; /* ID of a class. Search only in Objects of this class */
	var $triggerID = 0; /* ID of a document which to use for displaying thr detail page */
	var $casesensitive = false; /* set to true when a search should be case sensitive */
	var $languages = ''; //string of Languages, separated by ,
	var $objectseourls = false;
	var $hidedirindex = false;

	/**
	 *
	 * @desc    constructor of class
	 *
	 * @param   name         string - name of listview
	 * @param   rows          integer - number of rows to display per page
	 * @param   offset        integer - start offset of first page
	 * @param   order         string - field name(s) to order by
	 * @param   desc          boolean - set to true, if order should be descendend
	 * @param   docType       string - doctype
	 * @param   class         integer - ID of a class. Search only in Objects of this class
	 * @param   cats          string - comma separated categories
	 * @param   catOr         boolean - set to true if it should be an 'OR condition'
	 * @param   casesensitive boolean - set to true when a search should be case sensitive
	 * @param   workspaceID   array - commaseperated list of id's of workspace
	 * @param   cols   		  integer - to display a table this is the number of cols
	 *
	 */
	function __construct($name, $rows, $offset, $order, $desc, $docType, $class, $cats, $catOr, $casesensitive, array $workspaceID, $triggerID, $cols, $customerFilterType, $languages, $hidedirindex, $objectseourls){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, $workspaceID, $cols, '', '', '', '', '', $customerFilterType);

		$this->triggerID = $triggerID;
		$this->objectseourls = $objectseourls;
		$this->hidedirindex = $hidedirindex;
		$this->languages = $languages;
		$this->docType = trim($docType);
		$this->class = intval($class);
		$this->casesensitive = $casesensitive;

		$where_lang = ($this->languages ? ' AND i.Language IN ("' . implode('","', array_map('escape_sql_query', array_filter(array_map('trim', explode(',', $this->languages))))) . '") ' : '');

		// correct order
		$orderArr = [];

		$orderArr1 = array_map('trim', explode(',', $this->order));
		$random = (in_array(self::RANDOM_FIELD, $orderArr1));
		$orderArr1 = $random ? [] : $orderArr1;

		foreach($orderArr1 as $o){
			if(trim($o)){
				$foo = preg_split('/ +/', $o);
				$oname = $foo[0];
				$otype = isset($foo[1]) ? $foo[1] : '';
				$orderArr[] = ['oname' => $oname, 'otype' => $otype];
			}
		}
		$this->order = '';
		foreach($orderArr as $o){
			switch(strtolower($o['oname'])){
				case self::DATECREATION_FIELD:
					$this->order .= 'COALESCE(f.CreationDate' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.CreationDate' : '') . ')' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case self::DATEMOD_FIELD:
					$this->order .= 'COALESCE(f.ModDate' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.ModDate' : '') . ')' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case self::FILENAME_FIELD:
					$this->order .= 'COALESCE(f.Text' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.Text' : '') . ')' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case self::PATH_FIELD:
				case 'path':
					$this->order .= 'COALESCE(f.Path' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.Path' : '') . ')' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case 'workspace':
					$this->order .= 'wsp.Path' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case self::ID_FIELD:
				case 'oid':
				case 'did':
				case 'id':
					$this->order .= 'ID' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case 'title':
				case 'text':
				case 'description':
					$this->order .= $o['oname'] . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
			}
		}
		$this->order = rtrim($this->order, ',');

		/* if($this->order && $this->desc && (!preg_match('|.+ desc$|i', $this->order))){
		  $this->order .= ' DESC';
		  } */


		$this->search = $this->DB_WE->escape($this->search);

		$cat_tail = ($this->cats ? we_category_category::getCatSQLTail($this->cats, 'i', $this->catOr, $this->DB_WE) : '');
		$dt = ($this->docType ? f('SELECT ID FROM ' . DOC_TYPES_TABLE . ' WHERE IsFolder=0 AND DocType LIKE "' . $this->DB_WE->escape($this->docType) . '"', '', $this->DB_WE) : 0);

		if($dt && $this->class){
			$dtcl_query = ' AND (i.Doctype="' . $this->DB_WE->escape($dt) . '" OR i.ClassID=' . $this->class . ') ';
		} else if($dt){
			$dtcl_query = ' AND i.Doctype="' . $this->DB_WE->escape($dt) . '" ';
		} else if($this->class){
			$dtcl_query = ' AND i.ClassID=' . $this->class . ' ';
		} else {
			$dtcl_query = '';
		}

		//FIXME: use fulltext index: MATCH(Text) AGAINST([+-]words* IN BINARY MODE)

		$spalte = ($this->casesensitive ? 'BINARY ' : '') . 'i.Text';
		$bOR = $bAND = [];
		foreach(preg_split('/ +/', $this->search) as $v1){
			if(preg_match('|^[-\+]|', $v1)){
				$bAND[] = (preg_match('|^-|', $v1) ? 'NOT ' : '') .
						$spalte . ' LIKE "%' . preg_replace('|^[-\+]|', '', $v1) . '%"';
			} else {
				$bOR[] = $spalte . ' LIKE "%' . $v1 . '%"';
			}
		}

		if($bOR){
			$bAND[] = '(' . implode(' OR ', $bOR) . ')';
		}

		$bedingung_sql = '(' . implode(' AND ', $bAND) . ')';
		$ranking = '(ROUND(MATCH(i.`Text`,i.`Title`,i.`Description`,i.`Keywords`) AGAINST("' . str_replace(['+', '-'], '', $this->search) . '"),3))';
		$ws_where = '';
		if($this->workspaceID){
			$allWsp = pushChildsFromArr($this->workspaceID, FILE_TABLE, true);
			$cond = ['i.WorkspaceID IN (' . implode(',', $allWsp) . ')'];
			$ws_where = ' AND (' . implode(' OR ', $cond) . ')';
		}
		$weDocumentCustomerFilter_tail = (we_modules_info::isActive(we_modules_info::CUSTOMER) ?
				we_customer_documentFilter::getConditionForListviewQuery($this->customerFilterType, $this) :
				'');
//FIXME: remove wsp, if possible
		$where = ' WHERE ' . $bedingung_sql . ' ' . $dtcl_query . ($cat_tail ? ' AND ' . $cat_tail : '') . $ws_where . ' ' . $where_lang . ' ' . $weDocumentCustomerFilter_tail;
		$this->anz_all = f('SELECT COUNT(DISTINCT i.ID,i.ClassID) FROM ' . INDEX_TABLE . ' i LEFT JOIN ' . FILE_TABLE . ' wsp ON wsp.ID=i.WorkspaceID ' . $where, '', $this->DB_WE);

		$this->DB_WE->query('SELECT
	i.Category,
	i.ID,
	i.ID AS DID,
	i.ID AS OID,
	i.ClassID,
	i.ClassID AS `' . we_listview_base::CLASS_FIELD . '`,
	i.Text,
	COALESCE(wsp.Path,"/") AS Workspace,
	i.WorkspaceID,
	i.Title,
	i.Description,
	COALESCE(f.Path' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.Path' : '') . ') AS Path,
	i.Language,
	' . ($random ? 'RAND() ' : $ranking) . ' AS ranking
FROM ' .
				INDEX_TABLE . ' i LEFT JOIN ' .
				FILE_TABLE . ' wsp ON wsp.ID=i.WorkspaceID LEFT JOIN ' .
				FILE_TABLE . ' f ON (i.ID=f.ID AND i.ClassID=0) ' .
				(we_modules_info::isActive(we_modules_info::OBJECT) ? 'LEFT JOIN ' .
						OBJECT_FILES_TABLE . ' `of` ON (i.ID=`of`.ID AND i.ClassID>0) ' : '') .
				$where . '
GROUP BY
	i.ClassID,
	i.ID
ORDER BY
	ranking DESC ' . ($this->order ? (',' . $this->order) : '') .
				(($this->maxItemsPerPage > 0) ? (' LIMIT ' . intval($this->start) . ',' . intval($this->maxItemsPerPage)) : ''));
		$this->anz = $this->DB_WE->num_rows();
	}

	public function next_record(){
		if($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$fileData = array_change_key_case(getHash('SELECT * FROM ' .
							($this->DB_WE->f('ClassID') ? OBJECT_FILES_TABLE : FILE_TABLE ) .
							' WHERE ID=' . intval($this->DB_WE->f('ID')) . ' LIMIT 1'));

			$this->Record = array_merge(array_change_key_case($this->DB_WE->getRecord()), self::preFixArrayKeys($fileData));

			if($this->Record['classid'] && $this->objectseourls && show_SeoLinks()){
				$objecttriggerid = ($this->triggerID ?: ($fileData ? $fileData['triggerid'] : 0));

				$path_parts = ($objecttriggerid ?
						pathinfo(id_to_path($objecttriggerid)) :
						pathinfo(getScriptName())
						);

				$pidstr = ($this->Record['workspaceid'] ? '?pid=' . intval($this->Record['workspaceid']) : '');

				if($this->hidedirindex && seoIndexHide($path_parts['basename'])){
					$this->Record[self::PATH_FIELD] = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') .
							(!empty($fileData['url']) ?
							'/' . $fileData['url'] . $pidstr :
							'/?' . WEOBJ_PARAM . '=' . $this->Record['id'] . str_replace('?', '&amp;', $pidstr));
				} else {
					$this->Record[self::PATH_FIELD] = (!empty($fileData['url']) ?
							($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . $path_parts['filename'] . '/' . $fileData['url'] . $pidstr :
							getScriptName() . '?' . WEOBJ_PARAM . '=' . $this->Record['id'] . str_replace('?', '&amp;', $pidstr));
				}
				$this->Record[self::TRIGGER_FIELD] = $objecttriggerid;
			}
			$this->count++;
			return true;
		}

		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}

		return false;
	}

	function f($key, $stripWE = false){
		return parent::f($key, $stripWE);
	}

	public function getCustomerRestrictionQuery($specificCustomersQuery, $classID, $mfilter, $listQuery){
		return 'FROM ' . CUSTOMER_FILTER_TABLE . ' cf WHERE cf.modelType!="' . we_base_ContentTypes::FOLDER . '" AND ' . $mfilter . ' AND (' . $listQuery . ' OR ' . $specificCustomersQuery . ')';
	}

}
