<?php

/**
 * webEdition CMS
 *
 * $Rev: 99f4118a6392 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-04 23:18 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 * @param       string  $type           Anmeldeverfahren, moegliche Werte sind: customer, csv, emailonly
 * @param       string  $fieldGroup     Erwartet eine Feldgruppe (Bereich) aus der webEdition KV; Default: "Newsletter"; Nur bei $type == customer
 * @param       string  $mailingList    Erwartet den Namen der Mailing-Liste OHNE Feldgruppe (Bereich) aus der webEdition KV; Default: "Ok"; Nur bei $type == customer
 */
abstract class we_tag_addDelNewsletterEmail extends we_weTag_tag{
	public static $isDoubleOptIn = false;
	public static $MAILING_LIST_EMPTY = true;

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, 'type', __CLASS__))){
			return $foo;
		}
		$useListsArray = we_base_request::_(we_base_request::BOOL, 'we_use_lists__');
		$confirmID = we_base_request::_(we_base_request::STRING, 'confirmID', 0);
		$subscribeMail = we_base_request::_(we_base_request::EMAIL, 'we_subscribe_email__');
		$isSubscribe = $subscribeMail || $confirmID;
		$unsubscribeMail = we_base_request::_(we_base_request::EMAIL, 'we_unsubscribe_email__');
		$isUnsubscribe = $unsubscribeMail != '';
		$doubleoptin = self::getAttribute('doubleoptin', $attribs, false, we_base_request::BOOL);

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$adminmailid = intval(self::getAttribute('adminmailid', $attribs, 0, we_base_request::INT));
		$adminsubject = self::getAttribute('adminsubject', $attribs, '', we_base_request::STRING);
		$adminemail = self::getAttribute('adminemail', $attribs, '', we_base_request::STRING);
		$fieldGroup = self::getAttribute('fieldGroup', $attribs, 'Newsletter', we_base_request::STRING);
		$abos = $paths = [];
		$db = new DB_WE();

		$customerFieldPrefs = we_newsletter_view::getSettings();

		if(!$useListsArray){
			switch($type){
				case 'customer':
					$tmpAbos = self::getAttribute('mailingList', $attribs, [], we_base_request::STRING_LIST);
					if(!$tmpAbos || $tmpAbos[0] == ''){
						$abos[0] = $fieldGroup . '_Ok';
					} else {// #6100
						foreach($tmpAbos as $abo){
							$abos[] = $fieldGroup . '_' . $abo;
						}
					}
					break;
				case 'csv':
					$paths = self::getAttribute('path', $attribs, ['newsletter.txt'], we_base_request::FILELISTA) ?: ['newsletter.txt'];
					break;
			}
		} elseif(isset($_REQUEST['we_subscribe_list__']) && is_array(($subList = we_base_request::_(we_base_request::HTML, 'we_subscribe_list__')))){
			switch($type){
				case 'customer':
					$tmpAbos = self::getAttribute('mailingList', $attribs, [], we_base_request::STRING_LIST);
					foreach($subList as $nr){
						$abos[] = $fieldGroup . '_' . $tmpAbos[intval($nr) - 1];
					}
					break;
				default:
					$tmpPaths = self::getAttribute('path', $attribs, [], we_base_request::FILELISTA);
					foreach($subList as $nr){
						$paths[] = $tmpPaths[intval($nr) - 1];
					}
					break;
			}
			if(empty($abos) && empty($paths)){
				self::$MAILING_LIST_EMPTY = true;
				if($isSubscribe){
					we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
				} else {
					we_newsletter_util::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
				}
				return '';
			}
		} else {
			self::$MAILING_LIST_EMPTY = true;
			if($isSubscribe){
				we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
			} else {
				we_newsletter_util::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
			}
			return '';
		}

		$db->query('DELETE FROM ' . NEWSLETTER_CONFIRM_TABLE . ' WHERE expire<NOW()');

		//NEWSLETTER SUBSCTIPTION
		if($isSubscribe){
			we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_SUCCESS;
			$err = we_newsletter_base::STATUS_SUCCESS;
			$f = we_newsletter_util::getNewsletterFields($confirmID, $err, we_base_request::_(we_base_request::EMAIL, 'mail', ''), $subscribeMail); //FIXME: use data from above
			// Setting Globals FOR WE-Tags
			we_newsletter_util::$MAIL = isset($f['subscribe_mail']) ? $f['subscribe_mail'] : '';
			we_newsletter_util::$SALUTATION = isset($f['subscribe_salutation']) ? $f['subscribe_salutation'] : '';
			we_newsletter_util::$TITLE = isset($f['subscribe_title']) ? $f['subscribe_title'] : '';
			we_newsletter_util::$FIRSTNAME = isset($f['subscribe_firstname']) ? $f['subscribe_firstname'] : '';
			we_newsletter_util::$LASTNAME = isset($f['subscribe_lastname']) ? $f['subscribe_lastname'] : '';
			if(!empty($f['lists'])){
				if(!we_modules_info::isActive(we_modules_info::CUSTOMER) || strpos($f['lists'], '.')){
					$paths = array_filter(explode(',', $f['lists']));
				} else {
					$abos = array_filter(explode(',', $f['lists']));
					$type = 'customer';
				}
			}

			if($err !== we_newsletter_base::STATUS_SUCCESS){
				we_newsletter_util::$WRITENEWSLETTER_STATUS = $err;
				return '';
			}
			if(empty($f)){
				we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
				return '';
			}

			if($doubleoptin && !$confirmID){ // Direkte Anmeldung mit doubleoptin => zuerst confirmmail verschicken.
				$includeimages = self::getAttribute('includeimages', $attribs, false, we_base_request::BOOL);
				$expires = self::getAttribute('expiredoubleoptin', $attribs, 1440, we_base_request::INT);
				$id = self::getAttribute('id', $attribs, 0, we_base_request::INT);
				$subject = self::getAttribute('subject', $attribs, 'newsletter', we_base_request::STRING);
				$from = self::getAttribute('from', $attribs, 'newsletter@' . $_SERVER['SERVER_NAME'], we_base_request::EMAIL);
				$toCC = self::getAttribute('recipientCC', $attribs, [], we_base_request::EMAILLISTA);
				$toBCC = self::getAttribute('recipientBCC', $attribs, [], we_base_request::EMAILLISTA);
				$mailid = self::getAttribute('mailid', $attribs, '', we_base_request::INT);
				we_newsletter_util::addDoubleOptIn($db, $type, empty($customerFieldPrefs['customer_email_field']) ? '' : $customerFieldPrefs['customer_email_field'], $f, $abos, $paths, $mailid, $includeimages, $expires, $id, $subject, $from, $toCC, $toBCC);
				return '';
			}
			//confirmID wurde übermittelt, eine Bestätigung liegt also vor
			switch($type){
				case 'customer':
					we_newsletter_util::addToDB($db, $customerFieldPrefs, $f, $abos);
					break;
				case 'emailonly':
					//nicht in eine Liste eintragen sondern adminmail versenden
					we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_SUCCESS;
					we_newsletter_util::mailNewSuccessfullNewsletterActiviation($adminmailid, $adminemail, $adminsubject, DEFAULT_CHARSET, $f, self::getAttribute('includeimages', $attribs, false, we_base_request::BOOL));
					break;
				case 'csv':
					//in die Liste eintragen
					we_newsletter_util::addToFile($paths, $f);
					if(we_newsletter_util::$WRITENEWSLETTER_STATUS === we_newsletter_base::STATUS_SUCCESS){
						we_newsletter_util::mailNewSuccessfullNewsletterActiviation($adminmailid, $adminemail, $adminsubject, DEFAULT_CHARSET, $f, self::getAttribute('includeimages', $attribs, false, we_base_request::BOOL));
					}
			}
			$db->query('DELETE FROM ' . NEWSLETTER_CONFIRM_TABLE . ' WHERE subscribe_mail="' . $db->escape(strtolower($f['subscribe_mail'])) . '"');
		}

		//NEWSLETTER UNSUBSCTIPTION
		if($isUnsubscribe){
			if(!we_newsletter_util::unsubscribeNewsletter($db, $type === 'customer', $customerFieldPrefs['customer_email_field'], $abos, $paths, $unsubscribeMail) && $type !== 'emailonly'){
				return '';
			}
			$f = we_newsletter_util::getNewsletterFields(0, $status, '', $unsubscribeMail);
			we_newsletter_util::mailNewSuccessfullNewsletterActiviation($adminmailid, $adminemail, $adminsubject, DEFAULT_CHARSET, $f, self::getAttribute('includeimages', $attribs, false, we_base_request::BOOL));
			we_newsletter_util::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_SUCCESS; // PATCH Zeile hinzugefügt
		}

		unset($_REQUEST['we_unsubscribe_email__'], $_REQUEST['we_subscribe_email__'], $_REQUEST['we_subscribe_html__'], $_REQUEST['we_subscribe_title__'], $_REQUEST['we_subscribe_salutation__'], $_REQUEST['we_subscribe_firstname__'], $_REQUEST['we_subscribe_lastname__'], $_REQUEST['we_subscribe_list__']);
		return '';
	}

}

function we_tag_addDelNewsletterEmail(array $attribs){
	return we_tag_addDelNewsletterEmail::tag($attribs);
}
