<?php

/**
 * webEdition CMS
 *
 * $Rev: eaa970c2c184 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-11 13:52 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_field extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(empty(we_tag_listview::$current)){
			return self::parseError(g_l('parser', '[field_not_in_lv]'), __CLASS__);
		}
		//show field in case of e.g. listdir
		if(we_tag_listview::$current instanceof stdClass){
			return we_tag_listview::$current->field;
		}
		$lvname = we_tag_listview::$current->name;

		$orgName = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$href = self::getAttribute('href', $attribs, '', we_base_request::URL);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$alt = self::getAttribute('alt', $attribs, '', we_base_request::STRING);
		$tid = self::getAttribute('tid', $attribs, 0, we_base_request::INT);
		$hyperlink = self::getAttribute('hyperlink', $attribs, false, we_base_request::BOOL);
		$id = self::getAttribute('id', $attribs, '', we_base_request::STRING);
		$xml = self::getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);
		$striphtml = self::getAttribute('striphtml', $attribs, false, we_base_request::BOOL);
		$only = self::getAttribute('only', $attribs, '', we_base_request::STRING);
		$usekey = self::getAttribute('usekey', $attribs, false, we_base_request::BOOL);
		$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
		$showpath = self::getAttribute('showpath', $attribs);
		$rootdir = self::getAttribute('rootdir', $attribs, '', we_base_request::FILE);
		$field = self::getAttribute('field', $attribs, '', we_base_request::STRING);
		$max = self::getAttribute('max', $attribs, 0, we_base_request::INT);
		$value = self::getAttribute('value', $attribs, '', we_base_request::RAW);
		$parentidname = self::getAttribute('parentidname', $attribs, 'we_parentid', we_base_request::STRING);
		$listviewname = self::getAttribute('listviewname', $attribs, $lvname, we_base_request::STRING);
		$src = self::getAttribute('src', $attribs, '', we_base_request::STRING);
		$target = self::getAttribute('target', $attribs, '', we_base_request::STRING);
		$autobr = self::getAttribute('autobr', $attribs, '', we_base_request::BOOL);

		$attribs['title'] = self::getAttribute('title', $attribs, '', we_base_request::STRING);
		$attribs['format'] = self::getAttribute('format', $attribs, '', we_base_request::STRING);
		$attribs['class'] = self::getAttribute('class', $attribs);
		$classid = self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$seeMode = self::getAttribute('seeMode', $attribs, true, we_base_request::BOOL);
		$customerid = self::getAttribute('customerid', $attribs, 0, we_base_request::INT);
		$attribs['country'] = self::getAttribute('country', $attribs, '', we_base_request::STRING);
		//zusatz typen für verchiedene field-Typen
		$thumbnail = self::getAttribute('thumbnail', $attribs, '', we_base_request::STRING);
		$thumbnailid = self::getAttribute('thumbnailid', $attribs, 0, we_base_request::INT);
// Image
		$attribs['classes'] = self::getAttribute('classes', $attribs, '', we_base_request::STRING);
		//Wwysiwyg
		$attribs['commands'] = self::getAttribute('commands', $attribs, '', we_base_request::STRING);
		//Wwysiwyg
		$attribs['fontnames'] = self::getAttribute('fontnames', $attribs, '', we_base_request::STRING);

		$alt = (strtolower($alt) === strtolower(we_listview_base::PATH_FIELD) ? we_listview_base::PATH_FIELD : (strtolower($alt) === strtolower(we_listview_base::TEXT_FIELD) ? we_listview_base::TEXT_FIELD : $alt));
		$name = (strtolower($orgName) === strtolower(we_listview_base::PATH_FIELD) ? we_listview_base::PATH_FIELD : (strtolower($orgName) === strtolower(we_listview_base::TEXT_FIELD) ? we_listview_base::TEXT_FIELD : $name));

		$classid && $attribs['classid'] = $classid;
		$seeMode && $attribs['seeMode'] = $seeMode;
		$customerid && $attribs['customerid'] = $customerid;
		$thumbnail && $attribs['thumbnail'] = $thumbnail;
		$thumbnailid && $attribs['thumbnailid'] = $thumbnailid;

		$href && $attribs['href'] = $href;
		$type && $attribs['type'] = $type;
		$alt && $attribs['alt'] = $alt;
		$tid && $attribs['tid'] = $tid;
		$id && $attribs['id'] = $id;
		$attribs['xml'] = $xml;
		$only && $attribs['only'] = $only;
		$usekey && $attribs['usekey'] = $usekey;
		$triggerid && $attribs['triggerid'] = $triggerid;
		$showpath && $attribs['showpath'] = $showpath;
		$rootdir && $attribs['rootdir'] = $rootdir;
		$value && $attribs['value'] = $value;
		$target && $attribs['target'] = $target;
		$src && $attribs['src'] = $src;
		$field && $attribs['field'] = $field;
		unset($attribs['winprops']);

		$out = '';

		//listview of documents, document with a block. Try to access by blockname.
		$name = (we_tag_listview::$current->f($name) ? $name : $orgName);

		$classid = (
			we_tag_listview::$current instanceof we_listview_objectBase ?
			we_tag_listview::$current->f(we_listview_base::CLASS_FIELD) :
			(we_tag_listview::$current instanceof we_listview_shopItems ?
			we_tag_listview::$current->f(we_listview_base::PROPPREFIX . 'tableid') :
			0
			)
			);

		$isImageDoc = (we_tag_listview::$current->f(we_listview_base::CONTENTTYPE_FIELD) == we_base_ContentTypes::IMAGE);
		$cal = we_tag_listview::$current->getCalendarStruct();
		$isCalendar = (!empty($cal['calendar']) && we_tag_listview::$current->isCalendarField($type));

		if((!we_tag_listview::$current->f(we_listview_base::ID_FIELD) && empty($cal['calendar']))){
			return '';
		}

		switch($type){
			case 'binary' :
				$t = $GLOBALS['we_doc']->getFieldByVal(we_tag_listview::$current->f($name), $type, $attribs, false, $GLOBALS['we_doc']->ParentID, $GLOBALS['we_doc']->Path, $classid, 'listview');
				switch($only){
					case '':
					case 'name':
						$out = $t[0];
						break;
					case 'path':
						$out = $t[1];
						break;
					case 'parentpath':
						$out = $t[2];
						break;
					case 'filename':
						$out = $t[3];
						break;
					case 'extension':
						$out = $t[4];
						break;
					case 'filesize':
						$out = $t[5];
						break;
				}
				$href = ($href ?: $t[1]);
				break;
			case 'link' :
				if(we_tag_listview::$current){
					$out = $GLOBALS['we_doc']->getFieldByVal(we_tag_listview::$current->f($name), 'link', $attribs, false, $GLOBALS['we_doc']->ParentID, $GLOBALS['we_doc']->Path, $classid, 'listview');
					$href = ($href ?: $out);
					break;
				}
			case 'img' :
				if($src){

					$imgAtts = array_merge([
						'alt' => ' ', //  alt must be set
						'src' => $src,
						'xml' => $xml,
						], useAttribs($attribs, ['alt', 'width', 'height', 'border', 'hspace', 'align', 'vspace'])); //  use some atts form attribs array

					$out = getHtmlTag('img', $imgAtts);
					if(!$out){
						//we have no image, so we don't generate an link
						return '';
					}
					break;
				}
			//intentionally no break
			case 'int' :
			case 'date' :
			case 'float' :
			case 'checkbox' :
			case 'collection':
				$idd = ($isImageDoc && $type === 'img' ) ? we_tag_listview::$current->f(we_listview_base::ID_FIELD) : we_tag_listview::$current->f($name);
				$out = ($idd == 0 ? '' : $GLOBALS['we_doc']->getFieldByVal($idd, $type, $attribs, false, $GLOBALS['we_doc']->ParentID, $GLOBALS['we_doc']->Path, $classid, 'listview'));
				if($type === 'img' && empty($out)){
					return '';
				}
				break;
			case 'day' :
			case 'dayname' :
			case 'dayname_long' :
			case 'dayname_short' :
			case 'month' :
			case 'monthname' :
			case 'monthname_long' :
			case 'monthname_short' :
			case 'year' :
			case 'hour' :
			case 'week' :
				list(, $langcode) = self::getOutLang($attribs);
				$out = we_tag_listview::$current->getCalendarField($type, $langcode);
				break;

			case 'multiobject':
				$temp = we_unserialize(we_tag_listview::$current->f($name));
				$out = is_array($temp) ? (!empty($temp['objects']) ? implode(',', $temp['objects']) : implode(',', $temp)) : '';
				break;
			case 'country' :
				list(, $langcode) = self::getOutLang($attribs);
				$out = (WE_COUNTRIES_DEFAULT != '' && we_tag_listview::$current->f($name) === '--' ?
					WE_COUNTRIES_DEFAULT :
					CheckAndConvertISOfrontend(we_base_country::getTranslation(we_tag_listview::$current->f($name), we_base_country::TERRITORY, $langcode))
					);
				break;
			case 'language' :
				list(, $langcode) = self::getOutLang($attribs);
				$out = CheckAndConvertISOfrontend(we_base_country::getTranslation(we_tag_listview::$current->f($name), we_base_country::LANGUAGE, $langcode));
				break;
			case 'shopVat' :
				if(we_modules_info::isActive(we_modules_info::SHOP)){
					if(!we_shop_category::isCategoryMode()){
						$normVal = $GLOBALS['we_doc']->getFieldByVal(we_tag_listview::$current->f(we_shop_shop::VAT_FIELD_NAME), $type, $attribs, false, $GLOBALS['we_doc']->ParentID, $GLOBALS['we_doc']->Path, $classid, 'listview'); // war '$GLOBALS['lv']->getElement', getElemet gibt es aber nicht in LVs, gefunden bei #4648
						$out = we_shop_vats::getVatRateForSite($normVal);
					} else {
						$shopVatAttribs = array_merge($attribs, [
							'shopcategoryid' => we_tag_listview::$current->f(we_shop_shop::CATEGORY_FIELD_NAME),
							'wedoccategories' => we_tag_listview::$current->f(we_listview_base::CATEGORY_FIELD)
						]);
						unset($shopVatAttribs['type']);

						$out = we_tag('shopVat', $shopVatAttribs);
					}
				}
				break;
			case 'shopCategory' :
				if(we_modules_info::isActive(we_modules_info::SHOP) && we_tag_listview::$current){
					$id = we_tag_listview::$current->f(we_shop_shop::CATEGORY_FIELD_NAME);
					$wedocCategory = we_tag_listview::$current->f(we_listview_base::CATEGORY_FIELD);

					$out = we_shop_category::getShopCatFieldByID($id, $wedocCategory, $field, $showpath, $rootdir, true);
				}
				break;
			case 'href' ://#6329: fixed for lv type=document. check later for other types! #6421: field type=href in we:block
				if(we_tag_listview::$current){
					switch(true){
						case (we_tag_listview::$current instanceof we_listview_document):
							$hrefArr = [
								'int' => we_tag_listview::$current->f($name . we_base_link::MAGIC_INT_LINK) ?: we_tag_listview::$current->f(self::getPostName($name) . we_base_link::MAGIC_INT_LINK),
								'intID' => we_tag_listview::$current->f($name . we_base_link::MAGIC_INT_LINK_ID) ?: we_tag_listview::$current->f(self::getPostName($name) . we_base_link::MAGIC_INT_LINK_ID),
								'extPath' => we_tag_listview::$current->f($name)
							];
							break;
						case (we_tag_listview::$current instanceof we_listview_objectBase):
						case (we_tag_listview::$current instanceof we_listview_shopOrderitem): //Fix #7816
							$hrefArr = we_unserialize(we_tag_listview::$current->f($name));
							if(!is_array($hrefArr)){
								$hrefArr = [];
							}
							break;
					}
					$out = $hrefArr ? we_contents_document::getHrefByArray($hrefArr) : '';
					$path_parts = pathinfo($out);
					if(!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename'])){
						$out = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
					}

					break;
				}
			default: // FIXME: treat type="select" as separate case, and clean up the mess with all this little fixes
				if($orgName === we_listview_base::PATH_FIELD && $triggerid && in_array(get_class(we_tag_listview::$current), ['we_listview_search', 'we_listview_object', 'we_listview_multiobject'])){
					$triggerpath = id_to_path($triggerid);
					$triggerpath_parts = pathinfo($triggerpath);
					$normVal = ($triggerpath_parts['dirname'] != '/' ? $triggerpath_parts['dirname'] : '') . '/' .
						(!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($triggerpath_parts['basename']) ?
						'' : $triggerpath_parts['filename'] . '/' ) .
						we_tag_listview::$current->f(we_listview_base::URL_FIELD);
				} else {
					$testtype = ($type === 'select' && $usekey) ? 'text' : $type;
					switch(true){
						case (we_tag_listview::$current instanceof we_listview_objectBase):
							if($type === 'select'){// bugfix #6399
								$attribs['name'] = $attribs['_name_orig'];
							}
						default:
							$normVal = $GLOBALS['we_doc']->getFieldByVal(we_tag_listview::$current->f($name), $testtype, $attribs, false, $GLOBALS['we_doc']->ParentID, $GLOBALS['we_doc']->Path, $classid, 'listview');
							$normVal = we_wysiwyg_editor::parseGalleryTags($normVal);

							if($orgName === we_listview_base::PATH_FIELD){
								$path_parts = pathinfo($normVal);
								if(!$GLOBALS['we']['main']['doc']->InWebEdition && !empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename'])){
									$normVal = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
								}
							}
					}
				}
				// bugfix 7557
				// wenn die Abfrage im Aktuellen Objekt kein Ergebnis liefert
				// wird in den eingebundenen Objekten ueberprueft ob das Feld existiert

				if($type === 'select' && $normVal === ''){
					//FIXME: remove getDBRecord
					$dbRecord = array_keys(we_tag_listview::$current->getDBRecord()); // bugfix #6399
					foreach($dbRecord as $glob_key){
						if(substr($glob_key, 0, 7) === we_object::QUERY_PREFIX){
							$normVal = $GLOBALS['we_doc']->getFieldByVal(we_tag_listview::$current->f($name), ($usekey ? 'text' : 'select'), $attribs, false, $GLOBALS['we_doc']->ParentID, $GLOBALS['we_doc']->Path, substr($glob_key, 7), 'listview'); // war '$GLOBALS['lv']->getElement', getElemet gibt es aber nicht in LVs, gefunden bei #4648
						}

						if($normVal != ''){
							break;
						}
					}
				}
				// EOF bugfix 7557

				if($name && $name != we_listview_base::HREF_FIELD){
					if($normVal === ''){
						$altVal = $GLOBALS['we_doc']->getFieldByVal(we_tag_listview::$current->f($alt), $type, $attribs, false, $GLOBALS['we_doc']->ParentID, $GLOBALS['we_doc']->Path, $classid, 'listview');
						if($altVal === ''){
							return '';
						}

						if($alt === we_listview_base::PATH_FIELD){
							$path_parts = pathinfo($altVal);
							if(!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename'])){
								$altVal = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
							}
						}
						$normVal = $altVal;
					}
					$out = self::cutText(($striphtml ? strip_tags($normVal) : $normVal), $max, $striphtml);
				} elseif($value){
					$out = ($striphtml ? strip_tags($value) : $value);
				} else if($striphtml){
					$out = strip_tags($out);
				}

				if($autobr){
					$out = nl2br($out);
				}
		}

		if($hyperlink || $name === we_listview_base::HREF_FIELD){
			$linkAttribs = useAttribs($attribs, ['xml', 'class', 'style']);
			if($target){ //  save atts in array
				$linkAttribs['target'] = $target;
			}

			foreach($attribs as $key => $val){
				if(strpos($key, 'pass_') === 0){
					$linkAttribs[$key] = $val;
				}
			}

			if($href){
				$linkAttribs['href'] = $href;
				return getHtmlTag('a', $linkAttribs, $out, true);
			}

			if($id && $isCalendar){
				$cal = we_tag_listview::$current->getCalendarStruct();
				if(!empty($cal['storage'])){

					foreach($cal['storage'] as $date){
						if((($cal['calendarCount'] > 0 || ($cal['calendar'] === 'day' && $cal['calendarCount'] >= 0)) && $cal['calendarCount'] <= $cal['numofentries']) && (intval($date) >= intval($cal['start_date']) && intval($date) <= intval($cal['end_date']))){
							$show = ($cal['calendar'] === 'year' ? 'month' : 'day');
							$linkAttribs['href'] = id_to_path($id) . '?' . http_build_query(array_merge(
										we_tag_listview::$current->getListviewRequest($listviewname), array_filter([
									'we_lv_calendar_' . $listviewname => $show,
									'we_lv_datefield_' . $listviewname => $cal['datefield'],
									'we_lv_date_' . $listviewname => ($cal['date'] >= 0 ? date('Y-m-d', $cal['date']) : '')
							])));

							return getHtmlTag('a', $linkAttribs, $out, true);
						}
					}
				}
			} elseif($id && $isImageDoc){
				$linkAttribs['href'] = id_to_path($id) . '?' . http_build_query(array_merge(
							we_tag_listview::$current->getListviewRequest($listviewname), [
						'we_lv_start_' . $lvname => ((we_tag_listview::$current->count + we_tag_listview::$current->start) - 1),
						'we_lv_pend_' . $lvname => (we_tag_listview::$current->start + we_tag_listview::$current->anz),
						'we_lv_pstart_' . $lvname => (we_tag_listview::$current->start)
				]));

				return getHtmlTag('a', $linkAttribs, $out, true);
			}

			if((we_tag_listview::$current instanceof we_listview_search) && we_tag_listview::$current->f('ClassID')){
				if($tid){
					$data = [
						WEOBJ_PARAM => intval(we_tag_listview::$current->f('OID')),
						'we_objectTID' => $tid
					];

					$tail = '&amp;we_objectTID=' . $tid . '&amp;jwt=' . we_helpers_jwt::encode($data, sha1(SECURITY_ENCRYPTION_KEY));
				} else {
					$tail = '';
				}

				$path_parts = pathinfo(getScriptName());
				$isSeo = we_tag_listview::$current->objectseourls;
				if($isSeo){
					$h = getHash('SELECT  Url,TriggerID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval(we_tag_listview::$current->f('OID')));
					$objecttriggerid = $h['TriggerID'];
					$objecturl = $h['Url'] . '?';
					if($objecttriggerid){
						$path_parts = pathinfo(id_to_path($objecttriggerid));
					}
				} else {
					$objecturl = '?' . WEOBJ_PARAM . '=' . we_tag_listview::$current->f('OID') . '&';
				}

				$pidstr = '?pid=' . intval(we_tag_listview::$current->f('WorkspaceID'));
				$linkAttribs['href'] = (!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename']) ?
					($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' .
					$objecturl .
					$pidstr :
					($isSeo && $objecturl ?
					($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . $path_parts['filename'] . '/' . $objecturl :
					getScriptName() . $objecturl
					) . $pidstr);

				$linkAttribs['href'] .= $tail;

				return ($name === we_listview_base::HREF_FIELD ?
					$linkAttribs['href'] :
					getHtmlTag('a', $linkAttribs, $out, true) //  output of link-tag
					);
			}
			if((we_tag_listview::$current instanceof we_listview_category) && we_tag('ifHasChildren')){
				$linkAttribs['href'] = getScriptName() . '?' . $parentidname . '=' . we_tag_listview::$current->f('ID');

				return ($name === we_listview_base::HREF_FIELD ?
					$linkAttribs['href'] :
					getHtmlTag('a', $linkAttribs, $out, true) //  output of link-tag
					);
			}
			$showlink = false;
			switch(get_class(we_tag_listview::$current)){
				case 'we_listview_document':
				case 'we_listview_collectionItems':
					$triggerid = $triggerid ?: we_tag_listview::$current->triggerID;
					$tailOwnId = '?we_documentID=' . we_tag_listview::$current->f(we_listview_base::ID_FIELD);
				case '':
				case 'we_listview_search':
				case 'we_listview_variants':
				case 'we_listview_shopItems':
				case 'we_customertag':
				case 'we_listview_customer':
					$showlink = true;
					break;
				case 'we_listview_object':
					$triggerid = !empty(we_tag_listview::$current->objectseourls) ? we_tag_listview::$current->f('we_triggerid') : ($triggerid ?: we_tag_listview::$current->triggerID);
					$showlink = $tid || $triggerid || we_tag_listview::$current->f(we_listview_base::PROPPREFIX . 'templates') || we_tag_listview::$current->docID;
					$tailOwnId = '?' . WEOBJ_PARAM . '=' . we_tag_listview::$current->f('OF_ID');
					break;
				case 'we_listview_multiobject':
					$showlink = we_tag_listview::$current->f(we_listview_base::PROPPREFIX . 'templates') || we_tag_listview::$current->docID;
					break;
				case 'we_listview_shopOrder': //listview type="order"
					$showlink = !empty($triggerid);
					break;
				default:
					$showlink = false;
					break;
			}

			if(!$showlink){
				return $out;
			}

			$tail = ($tid && (we_tag_listview::$current instanceof we_listview_object) ? '&amp;we_objectTID=' . $tid : '');

			if(isset($GLOBALS['we_doc']->OF_ID) && ($GLOBALS['we_doc']->InWebEdition)){
				$linkAttribs['href'] = we_tag_listview::$current->f(we_listview_base::LASTPATH_FIELD) . $tail;
			} else {
				$path_parts = pathinfo(we_tag_listview::$current->f(we_listview_base::PATH_FIELD));
				if($triggerid){
					$triggerpath = id_to_path($triggerid);
					$triggerpath_parts = pathinfo($triggerpath);

					$linkAttribs['href'] = (!empty(we_tag_listview::$current->objectseourls)) ? // objectseourls=true
						(empty($triggerpath_parts['dirname']) ? '' : rtrim($triggerpath_parts['dirname'], '/') ) . '/' .
						((!$GLOBALS['we']['main']['doc']->InWebEdition && !empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($triggerpath_parts['basename'])) ? //hidedirindex of triggerID
						we_tag_listview::$current->f(we_listview_base::URL_FIELD) . $tail : //Fix #8708 do not hidedirindex of triggerID
						$triggerpath_parts['filename'] . '/' . we_tag_listview::$current->f(we_listview_base::URL_FIELD) . $tail
						) : //objectseourls=false or not set
						$triggerpath . $tailOwnId . $tail;

					/* End Fix '7771 */
				} else {
					$linkAttribs['href'] = (!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename']) ?
						($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
						we_tag_listview::$current->f(we_listview_base::PATH_FIELD) . $tail
						);
				}
			}

			return ($name === we_listview_base::HREF_FIELD ? //  return href for this object
				$linkAttribs['href'] :
				getHtmlTag('a', $linkAttribs, $out, true));
		}

		return $out;
	}

}

function we_tag_field(array $attribs){
	return we_tag_field::tag($attribs);
}
