<?php

/**
 * webEdition CMS
 *
 * $Rev: eb2b9b72567a $
 * $Author: Lukas Imhof $
 * $Date: 2019-11-07 13:14 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_img extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(!($tagId = self::getAttribute('id', $attribs, 0, we_base_request::INT)) && ($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
			return $foo;
		}

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$startid = self::getAttribute('startid', $attribs, 0, we_base_request::INT);
		$parentid = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
		$showcontrol = self::getAttribute('showcontrol', $attribs, true, we_base_request::BOOL);
		$showThumb = self::getAttribute('showthumbcontrol', $attribs, false, we_base_request::BOOL);
		$showimage = self::getAttribute('showimage', $attribs, true, we_base_request::BOOL);
		$showinputs = self::getAttribute('showinputs', $attribs, SHOWINPUTS_DEFAULT, we_base_request::BOOL);

		$tagAttribs = removeAttribs($attribs, ['id', 'showcontrol', 'showthumbcontrol', 'showimage', 'showinputs', 'startid', 'parentid']);
		$tagAttribs['thumbnail'] = self::getAttribute('thumbnail', $attribs, '', we_base_request::STRING);
		$tagAttribs['thumbnailid'] = self::getAttribute('thumbnailid', $attribs, 0, we_base_request::INT);

		if($name){
			$id = (($GLOBALS['we_doc']->getElement($name, we_contents_base::ETYPE_NUMBER) ?:
				$GLOBALS['we_doc']->getElement($name)) ?:
				$tagId);
		} else {
			$showThumb = $showcontrol = false;
			$id = $tagId;
		}

		//look if image exists in tblfile, and is an image
		if(!f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::IMAGE . '" AND ID=' . intval($id))){
			$id = 0;
		}

		$altField = $name . we_imageDocument::ALT_FIELD;
		$titleField = $name . we_imageDocument::TITLE_FIELD;
		$thumbField = $name . we_imageDocument::THUMB_FIELD;

		$fname = 'we_' . $GLOBALS['we_doc']->Name . '_img[' . $name . '#' . we_contents_base::ETYPE_NUMBER . ']';
		$altname = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $altField . ']';
		$titlename = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $titleField . ']';
		$thumbname = 'we_' . $GLOBALS['we_doc']->Name . '_thumb[' . $thumbField . '#' . we_contents_base::ETYPE_NUMBER . ']';

		if($id){
			$img = new we_imageDocument();
			$img->initByID($id);

			$tagAttribs['alt'] = self::getAttribute('alt', $attribs, $img->getElement('alt'), we_base_request::STRING);
			$tagAttribs['title'] = self::getAttribute('title', $attribs, $img->getElement('title'), we_base_request::STRING);
		}

		// images can now have custom attribs
		if(empty($GLOBALS['we']['editmode']) || !(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0) === 'reload_editpage' &&
			($name === we_base_request::_(we_base_request::HTML, 'we_cmd', '', 1)) &&
			we_base_request::_(we_base_request::STRING, 'we_cmd', '', 2) === 'change_image') &&
			$GLOBALS['we_doc']->issetElement($altField)){ // if no other image is selected.
			$alt = $GLOBALS['we_doc']->getElement($altField, we_contents_base::ETYPE_TEXT);
			$title = $GLOBALS['we_doc']->getElement($titleField, we_contents_base::ETYPE_TEXT);
			$tagAttribs['alt'] = $alt ?: (empty($tagAttribs['alt']) ? '' : $tagAttribs['alt']);
			$tagAttribs['title'] = $title ?: (empty($tagAttribs['title']) ? '' : $tagAttribs['title'] );
		} elseif(isset($GLOBALS['we_doc'])){
			$alt = $GLOBALS['we_doc']->getElement($altField, we_contents_base::ETYPE_TEXT);
			$title = $GLOBALS['we_doc']->getElement($titleField, we_contents_base::ETYPE_TEXT);
			if(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 2) === 'change_image'){
				//in case of changed images give them priority to already set text
				$tagAttribs['alt'] = (empty($tagAttribs['alt']) ? $alt : $tagAttribs['alt'] );
				$tagAttribs['title'] = (empty($tagAttribs['title']) ? $title : $tagAttribs['title'] );
			} else {
				$tagAttribs['alt'] = $alt ?: (empty($tagAttribs['alt']) ? '' : $tagAttribs['alt']);
				$tagAttribs['title'] = $title ?: (empty($tagAttribs['title']) ? '' : $tagAttribs['title'] );
			}
		}
		if($showThumb && isset($GLOBALS['we_doc'])){
			$thumbattr = $GLOBALS['we_doc']->getElement($thumbField, we_contents_base::ETYPE_NUMBER);
			$tagAttribs['thumbnailid'] = $thumbattr;
		}

		if(!empty($GLOBALS['we']['editmode']) && !$showimage){
			$out = '';
		} elseif($id){
			if($tagId){//bug 6433: später wird so ohne weiteres gar nicht mehr auf die id zurückgegriffen
				$tagAttribs['id'] = $tagId; //siehe korrespondierende Änderung in we:document::getField
				$tagAttribs['showcontrol'] = $showcontrol; //sicherstellen das es boolean iost
			}
			$out = $GLOBALS['we_doc']->getField($tagAttribs, 'img');
		} elseif($GLOBALS['we']['editmode'] && $GLOBALS['we_doc']->InWebEdition){
			$out = we_html_element::div(['class' => 'tag_img_icon largeicons grayicons',
					'data-contenttype' => we_base_ContentTypes::IMAGE]);
		} else {
			$out = '';
		}

		$cmd = 'tag_weimg_insertImage,' . $name . ',' . $fname . ',' . $GLOBALS['we_doc']->ID . ',' . (isset($GLOBALS['we_transaction']) ? $GLOBALS['we_transaction'] : '') . ',' . $GLOBALS['we_doc']->Table . ',' . $GLOBALS['we_doc']->EditPageNr;

		if(!empty($GLOBALS['we']['editmode']) && $out && $showcontrol){ //in editMode we surround image with dropzone
			$cmdTree = $cmdExt = 'tag_weimg_insertImage,' . $name . ',' . $fname . ',' . $GLOBALS['we_doc']->ID . ',' . (isset($GLOBALS['we_transaction']) ? $GLOBALS['we_transaction'] : '') . ',' . $GLOBALS['we_doc']->Table . ',' . $GLOBALS['we_doc']->EditPageNr;
			$out = we_fileupload_ui_base::getExternalDropZone($GLOBALS['we_doc']->Name . '_' . $name, $out, 'width:auto;height:auto;padding:0;', true, true, $cmdTree, $cmdExt);
		}

		if(!$id && (!$GLOBALS['we']['editmode'])){
			return '';
		}

		if($showcontrol && !empty($GLOBALS['we']['editmode'])){
			$id = $id ?: '';
			$out = '
<table class="we we_img weEditTable">
	<tr><td style="text-align:center">' . $out . '<input onchange="_EditorFrame.setEditorIsHot(true);" type="hidden" name="' . $fname . '" value="' . $id . '" /></td></tr>' .
				($showinputs ? //  only when wanted
				'
	<tr><td>
		<table class="weEditTable">
			<tr>
				<td>' . g_l('weClass', '[alt_kurz]') . ':&nbsp;</td>
				<td>' . we_html_tools::textInput($altname, $tagAttribs['alt'], '', ['onchange' => "_EditorFrame.setEditorIsHot(true);"]) . '</td>
			</tr>
			<tr>
				<td>' . g_l('weClass', '[Title]') . ':&nbsp;</td>
				<td>' . we_html_tools::textInput($titlename, $tagAttribs['title'], '', ['onchange' => "_EditorFrame.setEditorIsHot(true);"]) . '</td>
			</tr>
		</table></td></tr>' : ''
				);

			if($showThumb){ //  only when wanted
				$db = $GLOBALS['DB_WE'];
				$db->query('SELECT ID,Text,description FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text');
				if($db->num_rows()){
					$thumbnails = '<select name="' . $thumbname . '" onchange="top.we_cmd(\'reload_editpage\'); _EditorFrame.setEditorIsHot(true);">' .
						'<option title="' . $db->f('description') . '" value=""' . (($thumbattr == '') ? (' selected="selected"') : "") . '></option>';
					while($db->next_record()){
						$thumbnails .= '<option value="' . $db->f('ID') . '"' . (($thumbattr == $db->f('ID')) ? (' selected="selected"') : "") . '>' . $db->f('Text') . '</option>';
					}
					$thumbnails .= '</select>';
					$out .= '
	<tr><td>
		<table class="weEditTable">
			<tr>
				<td style="color: black; font-size: 12px;">' . g_l('weClass', '[thumbnails]') . ':&nbsp;</td>
				<td>' . $thumbnails . '</td>
			</tr>
		</table>
	</td></tr>';
				}
			}
			$out .= '
	<tr><td style="text-align:center">' .
				($id ?
				//	show edit_image_button
				//	we use hardcoded Content-Type - because it must be an image -> <we:img  >
				we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE,$id,WE().consts.contentTypes.IMAGE)") :
				// disable edit_image_button
				we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "#", '', "", true)
				) .
				we_html_button::create_button(we_html_button::IMAGE_SELECT, "javascript:we_cmd('we_selector_image', '" . ($id ?: $startid) . "', WE().consts.tables.FILE_TABLE,'" . $fname . "','','" . $cmd . "',''," . $parentid . ",WE().consts.contentTypes.IMAGE, " . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ")") .
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('remove_image', '" . $name . "')") .
				'</td></tr></table>';
		}
		return $out;
	}

}

function we_tag_img(array $attribs){
	return we_tag_img::tag($attribs);
}
