<?php

/**
 * webEdition CMS
 *
 * $Rev: 3514666a6c77 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-22 16:21 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_sessionField extends we_weTag_tag{
	const USED_FIELDS_NAME = '__usedField';

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
			return $foo;
		}

		$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$xml = self::getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);
		$removeFirstParagraph = self::getAttribute("removefirstparagraph", $attribs, defined('REMOVEFIRSTPARAGRAPH_DEFAULT') ? REMOVEFIRSTPARAGRAPH_DEFAULT : true, we_base_request::BOOL);
		//$autobrAttr = self::getAttribute('autobr', $attribs, false, we_base_request::BOOL);
		$checked = self::getAttribute('checked', $attribs, false, we_base_request::BOOL);
		$values = self::getAttribute('values', $attribs, [], we_base_request::STRING_LIST);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$size = self::getAttribute('size', $attribs, '', we_base_request::UNIT);
		$dateformat = self::getAttribute('dateformat', $attribs, '', we_base_request::RAW);
		$value = self::getAttribute('value', $attribs, '', we_base_request::RAW);
		$orgVal = (isset($_SESSION['webuser'][$name]) && (strlen($_SESSION['webuser'][$name]) > 0)) ? $_SESSION['webuser'][$name] : (($type === 'radio') ? '' : $value);


		$autofill = self::getAttribute('autofill', $attribs, false, we_base_request::BOOL);
		if($autofill){
			$condition = ($name === 'Username' ?
				['caps' => 4, 'small' => 4, 'nums' => 4, 'specs' => 0] :
				['caps' => 3, 'small' => 4, 'nums' => 3, 'specs' => 2]);

			$pass = new we_customer_generatePassword(8, $condition);
			$orgVal = $pass->PassGen();
			//echo $tmppass;
		}
		$ret = we_html_element::hidden(self::USED_FIELDS_NAME . '[]', $name);
		switch($type){
			case 'date' :
				$currentdate = self::getAttribute('currentdate', $attribs, false, we_base_request::BOOL);
				$minyear = self::getAttribute('minyear', $attribs, 0, we_base_request::INT);
				$maxyear = self::getAttribute('maxyear', $attribs, 0, we_base_request::INT);
				if($currentdate){
					$orgVal = time();
				}
				try{
					$date = ($orgVal ? new DateTime((is_numeric($orgVal) ? '@' : '') . $orgVal) : 0);
				} catch (Exception $e){
					$date = new DateTime('now');
				}
				list(, $langcode) = self::getOutLang($attribs);

				return $ret . we_html_tools::getDateInput('s[we_date_' . $name . ']', $date, false, $dateformat, '', '', $xml, $minyear, $maxyear, '', $langcode);

			case 'country':
				$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'name', 'value', 'values', 'onclick', 'onClick', 'mode', 'choice', 'pure', 'rows',
					'cols', 'maxlength', 'wysiwyg']);
				$newAtts['name'] = 's[' . $name . ']';
				list($lang, $langcode) = self::getOutLang($attribs);

				$topCountries = WE_COUNTRIES_TOP ? array_flip(explode(',', WE_COUNTRIES_TOP)) : [];
				foreach($topCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
				}

				$shownCountries = WE_COUNTRIES_SHOWN ? array_flip(explode(',', WE_COUNTRIES_SHOWN)) : [];
				foreach($shownCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $lang . '.UTF-8');
				asort($topCountries, SORT_LOCALE_STRING);
				asort($shownCountries, SORT_LOCALE_STRING);
				setlocale(LC_ALL, $oldLocale);

				$content = '';
				if(WE_COUNTRIES_DEFAULT != ''){
					$content .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
				}
				foreach($topCountries as $countrykey => $countryvalue){
					$content .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue) . '</option>';
				}

				if($topCountries && $shownCountries){
					$content .= '<option value="-" disabled="disabled">----</option>';
				}
				foreach($shownCountries as $countrykey => $countryvalue){
					$content .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue) . '</option>';
				}

				return $ret . getHtmlTag('select', $newAtts, $content, true);

			case 'language':
				$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
					'choice', 'pure', 'rows', 'cols', 'maxlength', 'wysiwyg']);
				$newAtts['name'] = 's[' . $name . ']';
				list($lang, $langcode) = self::getOutLang($attribs);

				$frontendL = $GLOBALS['we']['lang']['FrontendLanguages'];
				foreach($frontendL as &$lcvalue){
					$lccode = explode('_', $lcvalue);
					$lcvalue = $lccode[0];
				}
				$frontendLL = [];
				foreach($frontendL as &$lcvalue){
					$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
				}

				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $lang . '.UTF-8');
				asort($frontendLL, SORT_LOCALE_STRING);
				setlocale(LC_ALL, $oldLocale);
				$content = '';
				foreach($frontendLL as $langkey => &$langvalue){
					$content .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($langvalue) . '</option>' . "\n";
				}
				unset($langvalue);
				return $ret . getHtmlTag('select', $newAtts, $content, true);

			case 'select':
				$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
					'choice', 'pure', 'rows', 'cols', 'maxlength', 'wysiwyg']);
				if(!$values){
					$regs = [];
					preg_match_all("/'(.*?)'/", $GLOBALS['DB_WE']->getColTyp(CUSTOMER_TABLE, $name), $regs);
					$values = $regs[1];
				}
				return $ret . self::selectField('s[' . $name . ']', $orgVal, $values, $newAtts, true);

			case 'choice':
				$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
					'choice', 'pure', 'maxlength', 'rows', 'cols', 'wysiwyg']);
				if(!$values){
					$regs = [];
					preg_match_all("/'(.*?)'/", $GLOBALS['DB_WE']->getColTyp(CUSTOMER_TABLE, $name), $regs);
					$values = $regs[1];
				}
				return $ret . we_html_tools::inputChoiceField('s_' . $name, $orgVal, $values, $newAtts, self::getAttribute('mode', $attribs, '', we_base_request::STRING));

			case 'textinput':
				$choice = self::getAttribute('choice', $attribs, false, we_base_request::BOOL);
				$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
					'choice', 'pure', 'wysiwyg', 'rows', 'cols']);
				//FIXME: can't this be done by calling the 'choice' switch?
				if($choice){ // because of backwards compatibility
					$newAtts = removeAttribs($newAtts, ['maxlength']);
					$newAtts['name'] = 's[' . $name . ']';

					$optionsAr = self::getAttribute('options', $attribs, [], we_base_request::STRING_LIST);
					$isin = 0;
					$options = '';
					foreach($optionsAr as $cur){
						if($cur == $orgVal){
							$options .= getHtmlTag('option', ['value' => oldHtmlspecialchars($cur), 'selected' => 'selected'], $cur, true);
							$isin = true;
						} else {
							$options .= getHtmlTag('option', ['value' => oldHtmlspecialchars($cur)], $cur, true);
						}
					}
					if(!$isin){
						$options .= getHtmlTag('option', ['value' => oldHtmlspecialchars($orgVal), 'selected' => 'selected'], oldHtmlspecialchars($orgVal), true);
					}
					return $ret . getHtmlTag('select', $newAtts, $options, true);
				}
				return $ret . self::inputTextInputField('s[' . $name . ']', $orgVal, $newAtts);

			case 'textarea':
				//old Attribute
				$pure = self::getAttribute('pure', $attribs, false, we_base_request::BOOL);
				if($pure){
					$attribs = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
						'choice', 'pure', 'size', 'wysiwyg']);
					return $ret . self::textareaField('s[' . $name . ']', $orgVal, $attribs);
				}
				return $ret . (!defined('WE_TEXTAREA_JS') || define('WE_TEXTAREA_JS', 1) ?
					we_html_element::jsScript(JS_DIR . 'windows.js') : '') .
					we_html_forms::weTextarea('s[' . $name . ']', $orgVal, $attribs, false, '', false, false, $xml, $removeFirstParagraph, '');

			case 'radio':
				if((!isset($_SESSION['webuser'][$name])) && $checked){
					$orgVal = $value;
				}
				$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
					'choice', 'pure', 'rows', 'cols', 'wysiwyg']);

				return $ret . self::inputRadioField('s[' . $name . ']', $orgVal, $value, $newAtts);
			case 'checkbox':
				$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'name', 'value', 'values', 'onclick', 'onClick', 'mode', 'choice', 'pure', 'rows',
					'cols', 'wysiwyg']);

				if((!isset($_SESSION['webuser'][$name])) && $checked){
					$orgVal = 1;
				}
				return $ret . self::inputCheckboxField('s[' . $name . ']', $orgVal, $newAtts);
			case 'password':
				$newAtts = removeAttribs($attribs, ['checked', 'options', 'selected', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode', 'choice', 'pure',
					'rows',
					'cols', 'wysiwyg']);
				$newAtts['name'] = 's[' . $name . ']';
				$newAtts['maxlength'] = isset($newAtts['maxlength']) ? $newAtts['maxlength'] : we_users_user::MAX_PWD_LEN;
				$newAtts['value'] = !empty($_SESSION['webuser']['registered']) ? we_customer_customer::NOPWD_CHANGE : '';

				return $ret . getHtmlTag('input', $newAtts);
			case 'print':
				$ascountry = self::getAttribute('ascountry', $attribs, false, we_base_request::BOOL);
				$aslanguage = self::getAttribute('aslanguage', $attribs, false, we_base_request::BOOL);
				list($lang, $langcode) = self::getOutLang($attribs);

				if($ascountry || $aslanguage){
					return ($ascountry && $orgVal === '--' ? '' : CheckAndConvertISOfrontend(we_base_country::getTranslation($orgVal, ($ascountry ? we_base_country::TERRITORY : we_base_country::LANGUAGE), $langcode)));
				}
				if($dateformat){//FIXME: use document settings for dateformat => get locale - note DateTime doesn't use locale settings
					try{
						$date = new DateTime((is_numeric($orgVal) ? '@' : '') . $orgVal);
						//we need to set it explicitly
						$date->setTimezone(new DateTimeZone(date_default_timezone_get()));
						return we_base_country::dateformat($langcode, $date, $dateformat);
					} catch (Exception $e){
						//fallback to default return
					}
				}
				return self::getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL) ? oldHtmlspecialchars($orgVal) : $orgVal;
			case 'hidden':
				$usevalue = self::getAttribute('usevalue', $attribs, false, we_base_request::BOOL);
				$languageautofill = self::getAttribute('languageautofill', $attribs, false, we_base_request::BOOL);
				$v = ($usevalue ? $value : $orgVal);
				$hidden = ['type' => 'hidden',
					'name' => 's[' . $name . ']',
					'value' => self::getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL) ? oldHtmlspecialchars($v) : $v,
					'xml' => $xml];
				if($languageautofill){
					list(, $langcode) = self::getOutLang($attribs);
					$hidden['value'] = $langcode;
				}
				return getHtmlTag('input', array_merge(removeAttribs($attribs, ['doc', 'usevalue', 'languageautofill', 'htmlspecialchars', 'xml']), $hidden));
			case 'img':
				if(!isset($_SESSION['webuser']['imgtmp'])){
					$_SESSION['webuser']['imgtmp'] = [];
				}
				if(!isset($_SESSION['webuser']['imgtmp'][$name])){
					$_SESSION['webuser']['imgtmp'][$name] = [];
				}

				$_SESSION['webuser']['imgtmp'][$name]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
				$_SESSION['webuser']['imgtmp'][$name]['width'] = self::getAttribute('width', $attribs, 0, we_base_request::INT);
				$_SESSION['webuser']['imgtmp'][$name]['height'] = self::getAttribute('height', $attribs, 0, we_base_request::INT);
				$_SESSION['webuser']['imgtmp'][$name]['quality'] = self::getAttribute('quality', $attribs, 8, we_base_request::INT);
				$_SESSION['webuser']['imgtmp'][$name]['keepratio'] = self::getAttribute('keepratio', $attribs, true, we_base_request::BOOL);
				$_SESSION['webuser']['imgtmp'][$name]['maximize'] = self::getAttribute('maximize', $attribs, false, we_base_request::BOOL);
				$_SESSION['webuser']['imgtmp'][$name]['id'] = $orgVal ?: '';

				$foo = id_to_path($_SESSION['webuser']['imgtmp'][$name]['id']);
				if(!$foo){
					$_SESSION['webuser']['imgtmp'][$name]['id'] = 0;
				}

				$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
				$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
				$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
				$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
				$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
				$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::STRING);

				if($_SESSION['webuser']['imgtmp'][$name]['id']){
					$attribs['id'] = $_SESSION['webuser']['imgtmp'][$name];
				}

				unset($attribs['width']);
				unset($attribs['height']);

				$showcontrol = self::getAttribute('showcontrol', $attribs, true, we_base_request::BOOL);
				if($showcontrol){

					if(($foo = self::attributeMissing($attribs, 'parentid', __CLASS__))){
						return $foo;
					}
				}

				$imgId = $_SESSION['webuser']['imgtmp'][$name]['id'];

				$thumbnail = self::getAttribute('thumbnail', $attribs, '', we_base_request::STRING);
				$thumbid = self::getAttribute('thumbnailid', $attribs, 0, we_base_request::INT);
				if($thumbnail != ''){
					$attr['thumbnail'] = $thumbnail;
					$imgTag = $GLOBALS['we_doc']->getFieldByVal($imgId, 'img', $attr);
				} elseif($thumbid){
					$attr['thumbnailid'] = $thumbid;
					$imgTag = $GLOBALS['we_doc']->getFieldByVal($imgId, 'img', $attr);
				} else {
					$imgTag = $GLOBALS['we_doc']->getFieldByVal($imgId, 'img');
				}

				if($showcontrol){
					$checked = '';
					$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
					return '<table class="we we_sessionfield weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' .
						$imgTag . '
			<input type="hidden" name="s[' . $name . ']" value="' . $_SESSION['webuser']['imgtmp'][$name]['id'] . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<input name="WE_SF_IMG_DATA[' . $name . ']" type="file" accept="' . implode(',', we_base_ContentTypes::getRealContentTypes(we_base_ContentTypes::IMAGE)) . '"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . ' />
		</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;">
						<input style="border:0 solid black;" type="checkbox" id="WE_SF_DEL_CHECKBOX_' . $name . '" name="WE_SF_DEL_CHECKBOX_' . $name . '" value="1" ' . $checked . '/>
					</td>
					<td>
						<label for="WE_SF_DEL_CHECKBOX_' . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label>
					</td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
				}
				return ($imgId ? $imgTag : '');
		}
		return '';
	}

}

function we_tag_sessionField(array $attribs){
	return we_tag_sessionField::tag($attribs);
}
