<?php
/**
 * webEdition CMS
 *
 * $Rev: cc823e5005ff $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-28 18:01 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
we_modules_info::isActive(we_modules_info::SHOP);

abstract class we_tag_showShopItemNumber extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, 'shopname', __CLASS__))){
			return $foo;
		}

		$shopname = self::getAttribute('shopname', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('option', $attribs, false, we_base_request::BOOL) ? 'select' : '';
		$type = $type ?: (self::getAttribute('inputfield', $attribs, false, we_base_request::BOOL) ? 'textinput' : '');
		$type = $type ?: (self::getAttribute('type', $attribs, '', we_base_request::STRING));
		$xml = self::getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);
		$num_format = self::getAttribute('num_format', $attribs, '', we_base_request::STRING);
		$floatquantities = self::getAttribute('floatquantities', $attribs, false, we_base_request::BOOL);

		if(!isset($GLOBALS[$shopname]) || empty($GLOBALS[$shopname])){
			return self::parseError(sprintf(g_l('parser', '[missing_createShop]'), 'showShopItemNumber'), __CLASS__);
		}

		if(!we_tag_listview::$current || !we_tag_listview::$current instanceof we_listview_shopItems){
			return '';
		}

		$attr = removeAttribs($attribs, ['option', 'inputfield', 'type', 'start', 'stop', 'shopname', 'nameto', 'to', 'floatquantities', 'num_format']);

		// $type of the field
		$articleType = ( we_tag_listview::$current->f('OF_ID') ? we_shop_shop::OBJECT : we_shop_shop::DOCUMENT);

		$itemQuantity = $GLOBALS[$shopname]->getItemQuantity(we_tag_listview::$current->getShoppingCartKey());

		switch($type){
			case 'select':
				$start = self::getAttribute('start', $attribs, 0, we_base_request::INT);
				$stop = self::getAttribute('stop', $attribs, 10, we_base_request::INT);
				$step = self::getAttribute('step', $attribs, 1, we_base_request::INT);
				$stop = ($floatquantities ?
					( $stop > $itemQuantity ? $stop : $itemQuantity) :
					( intval($stop) > intval($itemQuantity) ? $stop : $itemQuantity));

				$attr['name'] = 'shop_cart_id[' . we_tag_listview::$current->getShoppingCartKey() . ']';
				$attr['size'] = 1;
				$attr['xml'] = $xml;

				$out = '';
				while($start <= $stop){
					$out .= ($itemQuantity == $start ?
						getHtmlTag('option', ['xml' => $xml, 'value' => $start, 'selected' => 'selected'], $start) :
						getHtmlTag('option', ['xml' => $xml, 'value' => $start], $start));

					$start = $start + $step;
				}
				return getHtmlTag('select', $attr, $out, true) . getHtmlTag('input', ['type' => 'hidden', 'name' => 't', 'value' => time()]);

			case 'textinput':
				$itemQuantity = ($floatquantities ? we_base_util::formatNumber($itemQuantity, $num_format, 2) : intval($itemQuantity));
				$attr = array_merge($attr, ['type' => 'text', 'name' => 'shop_cart_id[' . we_tag_listview::$current->getShoppingCartKey() . ']', 'size' => 2, 'value' => $itemQuantity]);
				return getHtmlTag('input', $attr) . getHtmlTag('input', ['type' => 'hidden', 'name' => 't', 'value' => time()]);

			default:
				return ($floatquantities ? we_base_util::formatNumber($itemQuantity, $num_format, 2) : intval($itemQuantity));
		}
	}

}

function we_tag_showShopItemNumber(array $attribs){
	return we_tag_showShopItemNumber::tag($attribs);
}
