<?php

/**
 * webEdition CMS
 *
 * $Rev: 3c4bd0bef8c6 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-29 10:31 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_votingField extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(!we_tag_voting::$currentVoting){
			return '';
		}

		$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$precision = self::getAttribute('precision', $attribs, 0, we_base_request::INT);
		$num_format = self::getAttribute('num_format', $attribs, '', we_base_request::STRING);
		$voting = we_tag_voting::$currentVoting;

		switch($name){
			case 'id':
				switch($type){
					case 'answer':
						return $voting->answerCount;
					case 'select':
						return '_we_voting_answer_' . $voting->ID;
					case 'radio':
					case 'checkbox':
					case 'chekbox':
						return '_we_voting_answer_' . $voting->ID . '_' . $voting->answerCount;
					case 'textarea':
					case 'textinput':
						return '_we_voting_answer_' . $voting->ID . '_' . $voting->answerCount;
					case 'voting':
					default:
						return $voting->ID;
				}
				break;
			case 'question':
				return stripslashes($voting->QASet[$voting->defVersion]['question']);
			case 'answer':
				switch($type){
					case 'radio':
						$code = '';
						$voting->IsRadio = true;
						$countanswers = count($voting->QASet[$voting->defVersion]['answers']);
						$subb = ($voting->AllowFreeText && $voting->answerCount == $countanswers - 1 ? 1 : 0);
						if($voting->answerCount < $countanswers - $subb){
							$atts = removeAttribs($attribs, ['name', 'type']);
							$atts['name'] = '_we_voting_answer_' . $voting->ID;
							if($voting->IsRequired && $voting->answerCount == 0){
								$atts['type'] = 'hidden';
								$code .= getHtmlTag('input', $atts, '');
							}
							$atts['id'] = '_we_voting_answer_' . $voting->ID . '_' . $voting->answerCount;
							$atts['value'] = $voting->answerCount;
							$atts['type'] = 'radio';
							if(isset($_SESSION['_we_voting_sessionData']) && isset($_SESSION['_we_voting_sessionData'][$voting->ID])){
								$selItem = $_SESSION['_we_voting_sessionData'][$voting->ID]['value'][0];
								if(is_numeric($selItem) && $selItem == $voting->answerCount){
									$atts['checked'] = 'checked';
								}
							}
							if($voting->AllowFreeText){
								$countanswers--;
								$atts['onclick'] = '_we_voting_answer_' . $voting->ID . '_' . $countanswers . '.value="";';
							}

							$code .= getHtmlTag('input', $atts, '');
						}
						return $code;
					case 'checkbox':
						$code = '';
						$voting->IsCheckbox = true;
						$countanswers = count($voting->QASet[$voting->defVersion]['answers']);
						$subb = ($voting->AllowFreeText && $voting->answerCount == $countanswers - 1 ? 1 : 0);
						if($voting->answerCount < $countanswers - $subb){
							$atts = removeAttribs($attribs, ['name', 'type']);
							$atts['name'] = '_we_voting_answer_' . $voting->ID;
							if($voting->IsRequired && $voting->answerCount == 0){
								$atts['type'] = 'hidden';
								$code .= getHtmlTag('input', $atts, '');
							}
							$atts['name'] = '_we_voting_answer_' . $voting->ID . '_' . $voting->answerCount;
							$atts['id'] = '_we_voting_answer_' . $voting->ID . '_' . $voting->answerCount;
							$atts['value'] = $voting->answerCount;
							$atts['type'] = 'checkbox';
							if(isset($_SESSION['_we_voting_sessionData']) && isset($_SESSION['_we_voting_sessionData'][$voting->ID])){
								foreach($_SESSION['_we_voting_sessionData'][$voting->ID]['value'] as $wert){
									$selItem = $wert;

									if(is_numeric($selItem) && $selItem == $voting->answerCount){
										$atts['checked'] = 'checked';
									}
								}
							}

							$code .= getHtmlTag('input', $atts, '');
						}
						return $code;
					case 'select':
						$code = '';
						if($voting->answerCount == 0){

							$atts = removeAttribs($attribs, ['name', 'type']);
							$atts['name'] = '_we_voting_answer_' . $voting->ID;
							$atts['id'] = '_we_voting_answer_' . $voting->ID;

							$code .= getHtmlTag('select', $atts, '');
						}

						$atts = removeAttribs($attribs, ['name', 'type']);
						$atts['value'] = $voting->answerCount;

						$code .= getHtmlTag('option', $atts, addslashes($voting->getAnswer()));
						if($voting->isLastSet()){
							$code .= '</select>';
						}
						return $code;
					case 'image':
						$countanswers = count($voting->QASet[$voting->defVersion]['answers']);
						if($voting->answerCount < $countanswers){
							$myImageID = stripslashes($voting->QASetAdditions[$voting->defVersion]['imageID'][$voting->answerCount]);
							if(is_numeric($myImageID)){
								$myImage = new we_imageDocument();
								$myImage->initByID($myImageID);

								$atts = removeAttribs($attribs, ['name', 'type', 'precision', 'num_format', 'nameto', 'to']);
								$myImage->initByAttribs($atts);
								return $myImage->getHtml();
							}
						}
						return '';
					case 'media':
						$countanswers = count($voting->QASet[$voting->defVersion]['answers']);
						if($voting->answerCount < $countanswers){
							$myMediaID = stripslashes($voting->QASetAdditions[$voting->defVersion]['mediaID'][$voting->answerCount]);
						}
						return id_to_path($myMediaID);
					case 'textinput':
						$code = '';
						if($voting->AllowFreeText){

							$atts = removeAttribs($attribs, ['name', 'type']);
							$countanswers = count($voting->QASet[$voting->defVersion]['answers']);
							if($voting->answerCount == $countanswers - 1){
								$atts['type'] = 'text';
								$atts['name'] = '_we_voting_answer_' . $voting->ID . '_' . $voting->answerCount;
								$atts['id'] = '_we_voting_answer_' . $voting->ID . '_' . $voting->answerCount;
								$value = '';
								if(isset($_SESSION['_we_voting_sessionData']) && isset($_SESSION['_we_voting_sessionData'][$voting->ID])){
									if($voting->IsRadio){
										$value = $_SESSION['_we_voting_sessionData'][$voting->ID]['value'][0];
									} else {
										if($voting->IsCheckbox){
											$mycount = count($_SESSION['_we_voting_sessionData'][$voting->ID]['value']);
											$value = $_SESSION['_we_voting_sessionData'][$voting->ID]['value'][$mycount - 1];
										} else {
											$value = $_SESSION['_we_voting_sessionData'][$voting->ID]['value'][$voting->answerCount];
										}
									}
								}
								if(!empty($voting->IsRadio)){
									$atts['onkeydown'] = '';
									for($i = 0; $i < $countanswers - 1; $i++){
										$atts['onkeydown'] .= '_we_voting_answer_' . $voting->ID . '_' . $i . '.checked=0;';
									}
								}
								$code .= getHtmlTag('input', $atts, $value);
							}
						}
						return $code;
					case 'textarea':
						if($voting->AllowFreeText){
							$atts = removeAttribs($attribs, ['name', 'type']);
							$countanswers = count($voting->QASet[$voting->defVersion]['answers']);
							if($voting->answerCount == $countanswers - 1){
								$atts['name'] = '_we_voting_answer_' . $voting->ID . '_' . $voting->answerCount;
								$atts['id'] = '_we_voting_answerfree_' . $voting->ID . '_' . $voting->answerCount;
								$value = '';
								if(isset($_SESSION['_we_voting_sessionData']) && isset($_SESSION['_we_voting_sessionData'][$voting->ID])){
									if($voting->IsRadio){
										$value = $_SESSION['_we_voting_sessionData'][$voting->ID]['value'][0];
									} else {
										if($voting->IsCheckbox){
											$mycount = count($_SESSION['_we_voting_sessionData'][$voting->ID]['value']);
											$value = $_SESSION['_we_voting_sessionData'][$voting->ID]['value'][$mycount - 1];
										} else {
											$value = $_SESSION['_we_voting_sessionData'][$voting->ID]['value'][$voting->answerCount];
										}
									}
								}
								if(!empty($voting->IsRadio)){
									$atts['onkeydown'] = '';
									for($i = 0; $i < $countanswers - 1; $i++){
										$atts['onkeydown'] .= '_we_voting_answer_' . $voting->ID . '_' . $i . '.checked=0;';
									}
								}
								return getHtmlTag('textarea', $atts, $value, true);
							}
						}
						return '';
					case 'text':
					default:
						$countanswers = count($voting->QASet[$voting->defVersion]['answers']);
						if($voting->answerCount < $countanswers){
							return stripslashes($voting->QASet[$voting->defVersion]['answers'][$voting->answerCount]);
						}
						return '';
				}
				break;
			case 'result':
				return $voting->getResult($type, $num_format, $precision);
			case 'date':
				$format = self::getAttribute('format', $attribs, '', we_base_request::STRING);
				return date(($format ?: g_l('weEditorInfo', '[date_format]')), $voting->PublishDate);
		}
	}

}

function we_tag_votingField(array $attribs){
	return we_tag_votingField::tag($attribs);
}
